#define HORZ_RES		32
#define VERT_RES		24
#define HORZ_CHR        8
#define VERT_CHR		10
#define VERT_FNT		8

#define HORZ_BAR        0x40
#define VERT_BAR        0x80

#define MARKER0_ACTIVE	0x03
#define MARKER1_ACTIVE	0x04
#define MARKER0_VERT	0x08
#define MARKER0_HORZ	0x09
#define MARKER1_VERT	0x0a
#define MARKER1_HORZ	0x0b

#define MARKER_HORZ_ADJ -1
#define MARKER_VERT_ADJ -10

