static INLINE void abx(void);
static INLINE void adca_di(void);
static INLINE void adca_ex(void);
static INLINE void adca_im(void);
static INLINE void adca_ix(void);
static INLINE void adcb_di(void);
static INLINE void adcb_ex(void);
static INLINE void adcb_im(void);
static INLINE void adcb_ix(void);
static INLINE void adda_di(void);
static INLINE void adda_ex(void);
static INLINE void adda_im(void);
static INLINE void adda_ix(void);
static INLINE void addb_di(void);
static INLINE void addb_ex(void);
static INLINE void addb_im(void);
static INLINE void addb_ix(void);
static INLINE void addd_di(void);
static INLINE void addd_ex(void);
static INLINE void addd_im(void);
static INLINE void addd_ix(void);
static INLINE void anda_di(void);
static INLINE void anda_ex(void);
static INLINE void anda_im(void);
static INLINE void anda_ix(void);
static INLINE void andb_di(void);
static INLINE void andb_ex(void);
static INLINE void andb_im(void);
static INLINE void andb_ix(void);
static INLINE void andcc(void);
static INLINE void asl_di(void);
static INLINE void asl_ex(void);
static INLINE void asl_ix(void);
static INLINE void asla(void);
static INLINE void aslb(void);
static INLINE void asr_di(void);
static INLINE void asr_ex(void);
static INLINE void asr_ix(void);
static INLINE void asra(void);
static INLINE void asrb(void);
static INLINE void bcc(void);
static INLINE void bcs(void);
static INLINE void beq(void);
static INLINE void bge(void);
static INLINE void bgt(void);
static INLINE void bhi(void);
static INLINE void bita_di(void);
static INLINE void bita_ex(void);
static INLINE void bita_im(void);
static INLINE void bita_ix(void);
static INLINE void bitb_di(void);
static INLINE void bitb_ex(void);
static INLINE void bitb_im(void);
static INLINE void bitb_ix(void);
static INLINE void ble(void);
static INLINE void bls(void);
static INLINE void blt(void);
static INLINE void bmi(void);
static INLINE void bne(void);
static INLINE void bpl(void);
static INLINE void bra(void);
static INLINE void brn(void);
static INLINE void bsr(void);
static INLINE void bvc(void);
static INLINE void bvs(void);
static INLINE void clr_di(void);
static INLINE void clr_ex(void);
static INLINE void clr_ix(void);
static INLINE void clra(void);
static INLINE void clrb(void);
static INLINE void cmpa_di(void);
static INLINE void cmpa_ex(void);
static INLINE void cmpa_im(void);
static INLINE void cmpa_ix(void);
static INLINE void cmpb_di(void);
static INLINE void cmpb_ex(void);
static INLINE void cmpb_im(void);
static INLINE void cmpb_ix(void);
static INLINE void cmpd_di(void);
static INLINE void cmpd_ex(void);
static INLINE void cmpd_im(void);
static INLINE void cmpd_ix(void);
static INLINE void cmps_di(void);
static INLINE void cmps_ex(void);
static INLINE void cmps_im(void);
static INLINE void cmps_ix(void);
static INLINE void cmpu_di(void);
static INLINE void cmpu_ex(void);
static INLINE void cmpu_im(void);
static INLINE void cmpu_ix(void);
static INLINE void cmpx_di(void);
static INLINE void cmpx_ex(void);
static INLINE void cmpx_im(void);
static INLINE void cmpx_ix(void);
static INLINE void cmpy_di(void);
static INLINE void cmpy_ex(void);
static INLINE void cmpy_im(void);
static INLINE void cmpy_ix(void);
static INLINE void com_di(void);
static INLINE void com_ex(void);
static INLINE void com_ix(void);
static INLINE void coma(void);
static INLINE void comb(void);
static INLINE void cwai(void);
static INLINE void daa(void);
static INLINE void dec_di(void);
static INLINE void dec_ex(void);
static INLINE void dec_ix(void);
static INLINE void deca(void);
static INLINE void decb(void);
static INLINE void eora_di(void);
static INLINE void eora_ex(void);
static INLINE void eora_im(void);
static INLINE void eora_ix(void);
static INLINE void eorb_di(void);
static INLINE void eorb_ex(void);
static INLINE void eorb_im(void);
static INLINE void eorb_ix(void);
static INLINE void exg(void);
static INLINE void illegal(void);
static INLINE void inc_di(void);
static INLINE void inc_ex(void);
static INLINE void inc_ix(void);
static INLINE void inca(void);
static INLINE void incb(void);
static INLINE void jmp_di(void);
static INLINE void jmp_ex(void);
static INLINE void jmp_ix(void);
static INLINE void jsr_di(void);
static INLINE void jsr_ex(void);
static INLINE void jsr_ix(void);
static INLINE void lbcc(void);
static INLINE void lbcs(void);
static INLINE void lbeq(void);
static INLINE void lbge(void);
static INLINE void lbgt(void);
static INLINE void lbhi(void);
static INLINE void lble(void);
static INLINE void lbls(void);
static INLINE void lblt(void);
static INLINE void lbmi(void);
static INLINE void lbne(void);
static INLINE void lbpl(void);
static INLINE void lbra(void);
static INLINE void lbrn(void);
static INLINE void lbsr(void);
static INLINE void lbvc(void);
static INLINE void lbvs(void);
static INLINE void lda_di(void);
static INLINE void lda_ex(void);
static INLINE void lda_im(void);
static INLINE void lda_ix(void);
static INLINE void ldb_di(void);
static INLINE void ldb_ex(void);
static INLINE void ldb_im(void);
static INLINE void ldb_ix(void);
static INLINE void ldd_di(void);
static INLINE void ldd_ex(void);
static INLINE void ldd_im(void);
static INLINE void ldd_ix(void);
static INLINE void lds_di(void);
static INLINE void lds_ex(void);
static INLINE void lds_im(void);
static INLINE void lds_ix(void);
static INLINE void ldu_di(void);
static INLINE void ldu_ex(void);
static INLINE void ldu_im(void);
static INLINE void ldu_ix(void);
static INLINE void ldx_di(void);
static INLINE void ldx_ex(void);
static INLINE void ldx_im(void);
static INLINE void ldx_ix(void);
static INLINE void ldy_di(void);
static INLINE void ldy_ex(void);
static INLINE void ldy_im(void);
static INLINE void ldy_ix(void);
static INLINE void leas(void);
static INLINE void leau(void);
static INLINE void leax(void);
static INLINE void leay(void);
static INLINE void lsr_di(void);
static INLINE void lsr_ex(void);
static INLINE void lsr_ix(void);
static INLINE void lsra(void);
static INLINE void lsrb(void);
static INLINE void mul(void);
static INLINE void neg_di(void);
static INLINE void neg_ex(void);
static INLINE void neg_ix(void);
static INLINE void nega(void);
static INLINE void negb(void);
static INLINE void nop(void);
static INLINE void ora_di(void);
static INLINE void ora_ex(void);
static INLINE void ora_im(void);
static INLINE void ora_ix(void);
static INLINE void orb_di(void);
static INLINE void orb_ex(void);
static INLINE void orb_im(void);
static INLINE void orb_ix(void);
static INLINE void orcc(void);
static INLINE void pshs(void);
static INLINE void pshu(void);
static INLINE void puls(void);
static INLINE void pulu(void);
static INLINE void rol_di(void);
static INLINE void rol_ex(void);
static INLINE void rol_ix(void);
static INLINE void rola(void);
static INLINE void rolb(void);
static INLINE void ror_di(void);
static INLINE void ror_ex(void);
static INLINE void ror_ix(void);
static INLINE void rora(void);
static INLINE void rorb(void);
static INLINE void rti(void);
static INLINE void rts(void);
static INLINE void sbca_di(void);
static INLINE void sbca_ex(void);
static INLINE void sbca_im(void);
static INLINE void sbca_ix(void);
static INLINE void sbcb_di(void);
static INLINE void sbcb_ex(void);
static INLINE void sbcb_im(void);
static INLINE void sbcb_ix(void);
static INLINE void sex(void);
static INLINE void sta_di(void);
static INLINE void sta_ex(void);
static INLINE void sta_im(void);
static INLINE void sta_ix(void);
static INLINE void stb_di(void);
static INLINE void stb_ex(void);
static INLINE void stb_im(void);
static INLINE void stb_ix(void);
static INLINE void std_di(void);
static INLINE void std_ex(void);
static INLINE void std_im(void);
static INLINE void std_ix(void);
static INLINE void sts_di(void);
static INLINE void sts_ex(void);
static INLINE void sts_im(void);
static INLINE void sts_ix(void);
static INLINE void stu_di(void);
static INLINE void stu_ex(void);
static INLINE void stu_im(void);
static INLINE void stu_ix(void);
static INLINE void stx_di(void);
static INLINE void stx_ex(void);
static INLINE void stx_im(void);
static INLINE void stx_ix(void);
static INLINE void sty_di(void);
static INLINE void sty_ex(void);
static INLINE void sty_im(void);
static INLINE void sty_ix(void);
static INLINE void suba_di(void);
static INLINE void suba_ex(void);
static INLINE void suba_im(void);
static INLINE void suba_ix(void);
static INLINE void subb_di(void);
static INLINE void subb_ex(void);
static INLINE void subb_im(void);
static INLINE void subb_ix(void);
static INLINE void subd_di(void);
static INLINE void subd_ex(void);
static INLINE void subd_im(void);
static INLINE void subd_ix(void);
static INLINE void swi(void);
static INLINE void swi2(void);
static INLINE void swi3(void);
static INLINE void sync(void);
static INLINE void tfr(void);
static INLINE void tst_di(void);
static INLINE void tst_ex(void);
static INLINE void tst_ix(void);
static INLINE void tsta(void);
static INLINE void tstb(void);

static INLINE void clrd(void); /* 6309 */
static INLINE void clrw_ix(void); /* 6309 ? */
static INLINE void clrw_di(void); /* 6309 ? */
static INLINE void clrw_ex(void); /* 6309 ? */
static INLINE void negd(void);
static INLINE void negw_ix(void); /* 6309 ? */
static INLINE void negw_di(void); /* 6309 ? */
static INLINE void negw_ex(void); /* 6309 ? */
static INLINE void lsrd( void ); /* 6309 */
static INLINE void lsrd_di( void ); /* 6309 */
static INLINE void lsrd_ix( void ); /* 6309 */
static INLINE void lsrd_ex( void ); /* 6309 */
static INLINE void rord( void ); /* 6309 ? */
static INLINE void rord_di( void ); /* 6309 */
static INLINE void rord_ix( void ); /* 6309 */
static INLINE void rord_ex( void ); /* 6309 */
static INLINE void asrd( void ); /* 6309 ? */
static INLINE void asrd_di( void ); /* 6309 */
static INLINE void asrd_ix( void ); /* 6309 */
static INLINE void asrd_ex( void ); /* 6309 */
static INLINE void asld( void ); /* 6309 */
static INLINE void asld_di( void ); /* 6309 */
static INLINE void asld_ix( void ); /* 6309 */
static INLINE void asld_ex( void ); /* 6309 */
static INLINE void rold( void ); /* 6309 ? */
static INLINE void rold_di( void ); /* 6309 */
static INLINE void rold_ix( void ); /* 6309 */
static INLINE void rold_ex( void ); /* 6309 */
static INLINE void tstd(void);
static INLINE void tstw_di( void );
static INLINE void tstw_ix( void );
static INLINE void tstw_ex( void );

/* Custom opcodes */
static INLINE void setline_im( void );
static INLINE void setline_ix( void );
static INLINE void setline_di( void );
static INLINE void setline_ex( void );
static INLINE void bmove( void );
static INLINE void move( void );
static INLINE void decbjnz( void );
static INLINE void decxjnz( void );
static INLINE void bset( void );
static INLINE void bset2( void );
static INLINE void lmul(void);
static INLINE void divx( void );
static INLINE void incd( void );
static INLINE void incw_di( void );
static INLINE void incw_ix( void );
static INLINE void incw_ex( void );
static INLINE void decd( void );
static INLINE void decw_di( void );
static INLINE void decw_ix( void );
static INLINE void decw_ex( void );
static INLINE void lsrw_di( void );
static INLINE void lsrw_ix( void );
static INLINE void lsrw_ex( void );
static INLINE void rorw_di( void );
static INLINE void rorw_ix( void );
static INLINE void rorw_ex( void );
static INLINE void asrw_di( void );
static INLINE void asrw_ix( void );
static INLINE void asrw_ex( void );
static INLINE void aslw_di( void );
static INLINE void aslw_ix( void );
static INLINE void aslw_ex( void );
static INLINE void rolw_di( void );
static INLINE void rolw_ix( void );
static INLINE void rolw_ex( void );
static INLINE void absa( void );
static INLINE void absb( void );
static INLINE void absd( void );

static INLINE void opcode2( void );

static void (*konami_main[0x100])(void) = {
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* 00 */
	opcode2,opcode2,opcode2,opcode2,pshs   ,pshu   ,puls   ,pulu   ,
	lda_im ,ldb_im ,opcode2,opcode2,adda_im,addb_im,opcode2,opcode2,	/* 10 */
	adca_im,adcb_im,opcode2,opcode2,suba_im,subb_im,opcode2,opcode2,
	sbca_im,sbcb_im,opcode2,opcode2,anda_im,andb_im,opcode2,opcode2,	/* 20 */
	bita_im,bitb_im,opcode2,opcode2,eora_im,eorb_im,opcode2,opcode2,
	ora_im ,orb_im ,opcode2,opcode2,cmpa_im,cmpb_im,opcode2,opcode2,	/* 30 */
	setline_im,opcode2,opcode2,opcode2,andcc,orcc  ,exg    ,tfr    ,
	ldd_im ,opcode2,ldx_im ,opcode2,ldy_im ,opcode2,ldu_im ,opcode2,	/* 40 */
	lds_im ,opcode2,cmpd_im,opcode2,cmpx_im,opcode2,cmpy_im,opcode2,
	cmpu_im,opcode2,cmps_im,opcode2,addd_im,opcode2,subd_im,opcode2,	/* 50 */
	opcode2,opcode2,opcode2,opcode2,opcode2,illegal,illegal,illegal,
	bra    ,bhi    ,bcc    ,bne    ,bvc    ,bpl    ,bge    ,bgt    ,	/* 60 */
	lbra   ,lbhi   ,lbcc   ,lbne   ,lbvc   ,lbpl   ,lbge   ,lbgt   ,
	brn    ,bls    ,bcs    ,beq    ,bvs    ,bmi    ,blt    ,ble    ,	/* 70 */
	lbrn   ,lbls   ,lbcs   ,lbeq   ,lbvs   ,lbmi   ,lblt   ,lble   ,
	clra   ,clrb   ,opcode2,coma   ,comb   ,opcode2,nega   ,negb   ,	/* 80 */
	opcode2,inca   ,incb   ,opcode2,deca   ,decb   ,opcode2,rts    ,
	tsta   ,tstb   ,opcode2,lsra   ,lsrb   ,opcode2,rora   ,rorb   ,	/* 90 */
	opcode2,asra   ,asrb   ,opcode2,asla   ,aslb   ,opcode2,rti    ,
	rola   ,rolb   ,opcode2,opcode2,opcode2,opcode2,opcode2,opcode2,	/* a0 */
	opcode2,opcode2,bsr    ,lbsr   ,decbjnz,decxjnz,nop    ,illegal,
	abx    ,daa	   ,sex    ,mul    ,lmul   ,divx   ,bmove  ,move   ,	/* b0 */
	lsrd   ,opcode2,rord   ,opcode2,asrd   ,opcode2,asld   ,opcode2,
	rold   ,opcode2,clrd   ,opcode2,negd   ,opcode2,incd   ,opcode2,	/* c0 */
	decd   ,opcode2,tstd   ,opcode2,absa   ,absb   ,absd   ,bset   ,
	bset2  ,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* d0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* e0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* f0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal
};

static void (*konami_indexed[0x100])(void) = {
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* 00 */
	leax   ,leay   ,leau   ,leas   ,illegal,illegal,illegal,illegal,
	illegal,illegal,lda_ix ,ldb_ix ,illegal,illegal,adda_ix,addb_ix,	/* 10 */
	illegal,illegal,adca_ix,adcb_ix,illegal,illegal,suba_ix,subb_ix,
	illegal,illegal,sbca_ix,sbcb_ix,illegal,illegal,anda_ix,andb_ix,	/* 20 */
	illegal,illegal,bita_ix,bitb_ix,illegal,illegal,eora_ix,eorb_ix,
	illegal,illegal,ora_ix ,orb_ix ,illegal,illegal,cmpa_ix,cmpb_ix,	/* 30 */
	illegal,setline_ix,sta_ix,stb_ix,illegal,illegal,illegal,illegal,
	illegal,ldd_ix ,illegal,ldx_ix ,illegal,ldy_ix ,illegal,ldu_ix ,	/* 40 */
	illegal,lds_ix ,illegal,cmpd_ix,illegal,cmpx_ix,illegal,cmpy_ix,
	illegal,cmpu_ix,illegal,cmps_ix,illegal,addd_ix,illegal,subd_ix,	/* 50 */
	std_ix ,stx_ix ,sty_ix ,stu_ix ,sts_ix ,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* 60 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* 70 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,clr_ix ,illegal,illegal,com_ix ,illegal,illegal,	/* 80 */
	neg_ix ,illegal,illegal,inc_ix ,illegal,illegal,dec_ix ,illegal,
	illegal,illegal,tst_ix ,illegal,illegal,lsr_ix ,illegal,illegal,	/* 90 */
	ror_ix ,illegal,illegal,asr_ix ,illegal,illegal,asl_ix ,illegal,
	illegal,illegal,rol_ix ,lsrw_ix,rorw_ix,asrw_ix,aslw_ix,rolw_ix,	/* a0 */
	jmp_ix ,jsr_ix ,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* b0 */
	illegal,lsrd_ix,illegal,rord_ix,illegal,asrd_ix,illegal,asld_ix,
	illegal,rold_ix,illegal,clrw_ix,illegal,negw_ix,illegal,incw_ix,	/* c0 */
	illegal,decw_ix,illegal,tstw_ix,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* d0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* e0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* f0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal
};

static void (*konami_direct[0x100])(void) = {
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* 00 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,lda_di ,ldb_di ,illegal,illegal,adda_di,addb_di,	/* 10 */
	illegal,illegal,adca_di,adcb_di,illegal,illegal,suba_di,subb_di,
	illegal,illegal,sbca_di,sbcb_di,illegal,illegal,anda_di,andb_di,	/* 20 */
	illegal,illegal,bita_di,bitb_di,illegal,illegal,eora_di,eorb_di,
	illegal,illegal,ora_di ,orb_di ,illegal,illegal,cmpa_di,cmpb_di,	/* 30 */
	illegal,setline_di,sta_di,stb_di,illegal,illegal,illegal,illegal,
	illegal,ldd_di ,illegal,ldx_di ,illegal,ldy_di ,illegal,ldu_di ,	/* 40 */
	illegal,lds_di ,illegal,cmpd_di,illegal,cmpx_di,illegal,cmpy_di,
	illegal,cmpu_di,illegal,cmps_di,illegal,addd_di,illegal,subd_di,	/* 50 */
	std_di ,stx_di ,sty_di ,stu_di ,sts_di ,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* 60 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* 70 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,clr_di ,illegal,illegal,com_di ,illegal,illegal,	/* 80 */
	neg_di ,illegal,illegal,inc_di ,illegal,illegal,dec_di ,illegal,
	illegal,illegal,tst_di ,illegal,illegal,lsr_di ,illegal,illegal,	/* 90 */
	ror_di ,illegal,illegal,asr_di ,illegal,illegal,asl_di ,illegal,
	illegal,illegal,rol_di ,lsrw_di,rorw_di,asrw_di,aslw_di,rolw_di,	/* a0 */
	jmp_di ,jsr_di ,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* b0 */
	illegal,lsrd_di,illegal,rord_di,illegal,asrd_di,illegal,asld_di,
	illegal,rold_di,illegal,clrw_di,illegal,negw_di,illegal,incw_di,	/* c0 */
	illegal,decw_di,illegal,tstw_di,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* d0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* e0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* f0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal
};

static void (*konami_extended[0x100])(void) = {
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* 00 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,lda_ex ,ldb_ex ,illegal,illegal,adda_ex,addb_ex,	/* 10 */
	illegal,illegal,adca_ex,adcb_ex,illegal,illegal,suba_ex,subb_ex,
	illegal,illegal,sbca_ex,sbcb_ex,illegal,illegal,anda_ex,andb_ex,	/* 20 */
	illegal,illegal,bita_ex,bitb_ex,illegal,illegal,eora_ex,eorb_ex,
	illegal,illegal,ora_ex ,orb_ex ,illegal,illegal,cmpa_ex,cmpb_ex,	/* 30 */
	illegal,setline_ex,sta_ex,stb_ex,illegal,illegal,illegal,illegal,
	illegal,ldd_ex ,illegal,ldx_ex ,illegal,ldy_ex ,illegal,ldu_ex ,	/* 40 */
	illegal,lds_ex ,illegal,cmpd_ex,illegal,cmpx_ex,illegal,cmpy_ex,
	illegal,cmpu_ex,illegal,cmps_ex,illegal,addd_ex,illegal,subd_ex,	/* 50 */
	std_ex ,stx_ex ,sty_ex ,stu_ex ,sts_ex ,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* 60 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* 70 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,clr_ex ,illegal,illegal,com_ex ,illegal,illegal,	/* 80 */
	neg_ex ,illegal,illegal,inc_ex ,illegal,illegal,dec_ex ,illegal,
	illegal,illegal,tst_ex ,illegal,illegal,lsr_ex ,illegal,illegal,	/* 90 */
	ror_ex ,illegal,illegal,asr_ex ,illegal,illegal,asl_ex ,illegal,
	illegal,illegal,rol_ex ,lsrw_ex,rorw_ex,asrw_ex,aslw_ex,rolw_ex,	/* a0 */
	jmp_ex ,jsr_ex ,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* b0 */
	illegal,lsrd_ex,illegal,rord_ex,illegal,asrd_ex,illegal,asld_ex,
	illegal,rold_ex,illegal,clrw_ex,illegal,negw_ex,illegal,incw_ex,	/* c0 */
	illegal,decw_ex,illegal,tstw_ex,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* d0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* e0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal,	/* f0 */
	illegal,illegal,illegal,illegal,illegal,illegal,illegal,illegal
};
