#ifndef __SHARED_H__
#define __SHARED_H__

#if defined(__cplusplus)
extern "C" {
#endif

extern const char* get_resource_path(char* file);
extern const char* get_documents_path(char* file);

extern char *IMAMEBASEPATH;
    
#if defined(__cplusplus)
}
#endif

#endif
