#ifndef M68KOPS__HEADER
#define M68KOPS__HEADER

/* ======================================================================== */
/* ============================ OPCODE HANDLERS =========================== */
/* ======================================================================== */


void m68k_op_1010(void);
void m68k_op_1111(void);
void m68k_op_abcd_rr_8(void);
void m68k_op_abcd_mm_8_ax7(void);
void m68k_op_abcd_mm_8_ay7(void);
void m68k_op_abcd_mm_8_axy7(void);
void m68k_op_abcd_mm_8(void);
void m68k_op_add_er_8_d(void);
void m68k_op_add_er_8_ai(void);
void m68k_op_add_er_8_pi(void);
void m68k_op_add_er_8_pi7(void);
void m68k_op_add_er_8_pd(void);
void m68k_op_add_er_8_pd7(void);
void m68k_op_add_er_8_di(void);
void m68k_op_add_er_8_ix(void);
void m68k_op_add_er_8_aw(void);
void m68k_op_add_er_8_al(void);
void m68k_op_add_er_8_pcdi(void);
void m68k_op_add_er_8_pcix(void);
void m68k_op_add_er_8_i(void);
void m68k_op_add_er_16_d(void);
void m68k_op_add_er_16_a(void);
void m68k_op_add_er_16_ai(void);
void m68k_op_add_er_16_pi(void);
void m68k_op_add_er_16_pd(void);
void m68k_op_add_er_16_di(void);
void m68k_op_add_er_16_ix(void);
void m68k_op_add_er_16_aw(void);
void m68k_op_add_er_16_al(void);
void m68k_op_add_er_16_pcdi(void);
void m68k_op_add_er_16_pcix(void);
void m68k_op_add_er_16_i(void);
void m68k_op_add_er_32_d(void);
void m68k_op_add_er_32_a(void);
void m68k_op_add_er_32_ai(void);
void m68k_op_add_er_32_pi(void);
void m68k_op_add_er_32_pd(void);
void m68k_op_add_er_32_di(void);
void m68k_op_add_er_32_ix(void);
void m68k_op_add_er_32_aw(void);
void m68k_op_add_er_32_al(void);
void m68k_op_add_er_32_pcdi(void);
void m68k_op_add_er_32_pcix(void);
void m68k_op_add_er_32_i(void);
void m68k_op_add_re_8_ai(void);
void m68k_op_add_re_8_pi(void);
void m68k_op_add_re_8_pi7(void);
void m68k_op_add_re_8_pd(void);
void m68k_op_add_re_8_pd7(void);
void m68k_op_add_re_8_di(void);
void m68k_op_add_re_8_ix(void);
void m68k_op_add_re_8_aw(void);
void m68k_op_add_re_8_al(void);
void m68k_op_add_re_16_ai(void);
void m68k_op_add_re_16_pi(void);
void m68k_op_add_re_16_pd(void);
void m68k_op_add_re_16_di(void);
void m68k_op_add_re_16_ix(void);
void m68k_op_add_re_16_aw(void);
void m68k_op_add_re_16_al(void);
void m68k_op_add_re_32_ai(void);
void m68k_op_add_re_32_pi(void);
void m68k_op_add_re_32_pd(void);
void m68k_op_add_re_32_di(void);
void m68k_op_add_re_32_ix(void);
void m68k_op_add_re_32_aw(void);
void m68k_op_add_re_32_al(void);
void m68k_op_adda_16_d(void);
void m68k_op_adda_16_a(void);
void m68k_op_adda_16_ai(void);
void m68k_op_adda_16_pi(void);
void m68k_op_adda_16_pd(void);
void m68k_op_adda_16_di(void);
void m68k_op_adda_16_ix(void);
void m68k_op_adda_16_aw(void);
void m68k_op_adda_16_al(void);
void m68k_op_adda_16_pcdi(void);
void m68k_op_adda_16_pcix(void);
void m68k_op_adda_16_i(void);
void m68k_op_adda_32_d(void);
void m68k_op_adda_32_a(void);
void m68k_op_adda_32_ai(void);
void m68k_op_adda_32_pi(void);
void m68k_op_adda_32_pd(void);
void m68k_op_adda_32_di(void);
void m68k_op_adda_32_ix(void);
void m68k_op_adda_32_aw(void);
void m68k_op_adda_32_al(void);
void m68k_op_adda_32_pcdi(void);
void m68k_op_adda_32_pcix(void);
void m68k_op_adda_32_i(void);
void m68k_op_addi_8_d(void);
void m68k_op_addi_8_ai(void);
void m68k_op_addi_8_pi(void);
void m68k_op_addi_8_pi7(void);
void m68k_op_addi_8_pd(void);
void m68k_op_addi_8_pd7(void);
void m68k_op_addi_8_di(void);
void m68k_op_addi_8_ix(void);
void m68k_op_addi_8_aw(void);
void m68k_op_addi_8_al(void);
void m68k_op_addi_16_d(void);
void m68k_op_addi_16_ai(void);
void m68k_op_addi_16_pi(void);
void m68k_op_addi_16_pd(void);
void m68k_op_addi_16_di(void);
void m68k_op_addi_16_ix(void);
void m68k_op_addi_16_aw(void);
void m68k_op_addi_16_al(void);
void m68k_op_addi_32_d(void);
void m68k_op_addi_32_ai(void);
void m68k_op_addi_32_pi(void);
void m68k_op_addi_32_pd(void);
void m68k_op_addi_32_di(void);
void m68k_op_addi_32_ix(void);
void m68k_op_addi_32_aw(void);
void m68k_op_addi_32_al(void);
void m68k_op_addq_8_d(void);
void m68k_op_addq_8_ai(void);
void m68k_op_addq_8_pi(void);
void m68k_op_addq_8_pi7(void);
void m68k_op_addq_8_pd(void);
void m68k_op_addq_8_pd7(void);
void m68k_op_addq_8_di(void);
void m68k_op_addq_8_ix(void);
void m68k_op_addq_8_aw(void);
void m68k_op_addq_8_al(void);
void m68k_op_addq_16_d(void);
void m68k_op_addq_16_a(void);
void m68k_op_addq_16_ai(void);
void m68k_op_addq_16_pi(void);
void m68k_op_addq_16_pd(void);
void m68k_op_addq_16_di(void);
void m68k_op_addq_16_ix(void);
void m68k_op_addq_16_aw(void);
void m68k_op_addq_16_al(void);
void m68k_op_addq_32_d(void);
void m68k_op_addq_32_a(void);
void m68k_op_addq_32_ai(void);
void m68k_op_addq_32_pi(void);
void m68k_op_addq_32_pd(void);
void m68k_op_addq_32_di(void);
void m68k_op_addq_32_ix(void);
void m68k_op_addq_32_aw(void);
void m68k_op_addq_32_al(void);
void m68k_op_addx_rr_8(void);
void m68k_op_addx_rr_16(void);
void m68k_op_addx_rr_32(void);
void m68k_op_addx_mm_8_ax7(void);
void m68k_op_addx_mm_8_ay7(void);
void m68k_op_addx_mm_8_axy7(void);
void m68k_op_addx_mm_8(void);
void m68k_op_addx_mm_16(void);
void m68k_op_addx_mm_32(void);
void m68k_op_and_er_8_d(void);
void m68k_op_and_er_8_ai(void);
void m68k_op_and_er_8_pi(void);
void m68k_op_and_er_8_pi7(void);
void m68k_op_and_er_8_pd(void);
void m68k_op_and_er_8_pd7(void);
void m68k_op_and_er_8_di(void);
void m68k_op_and_er_8_ix(void);
void m68k_op_and_er_8_aw(void);
void m68k_op_and_er_8_al(void);
void m68k_op_and_er_8_pcdi(void);
void m68k_op_and_er_8_pcix(void);
void m68k_op_and_er_8_i(void);
void m68k_op_and_er_16_d(void);
void m68k_op_and_er_16_ai(void);
void m68k_op_and_er_16_pi(void);
void m68k_op_and_er_16_pd(void);
void m68k_op_and_er_16_di(void);
void m68k_op_and_er_16_ix(void);
void m68k_op_and_er_16_aw(void);
void m68k_op_and_er_16_al(void);
void m68k_op_and_er_16_pcdi(void);
void m68k_op_and_er_16_pcix(void);
void m68k_op_and_er_16_i(void);
void m68k_op_and_er_32_d(void);
void m68k_op_and_er_32_ai(void);
void m68k_op_and_er_32_pi(void);
void m68k_op_and_er_32_pd(void);
void m68k_op_and_er_32_di(void);
void m68k_op_and_er_32_ix(void);
void m68k_op_and_er_32_aw(void);
void m68k_op_and_er_32_al(void);
void m68k_op_and_er_32_pcdi(void);
void m68k_op_and_er_32_pcix(void);
void m68k_op_and_er_32_i(void);
void m68k_op_and_re_8_ai(void);
void m68k_op_and_re_8_pi(void);
void m68k_op_and_re_8_pi7(void);
void m68k_op_and_re_8_pd(void);
void m68k_op_and_re_8_pd7(void);
void m68k_op_and_re_8_di(void);
void m68k_op_and_re_8_ix(void);
void m68k_op_and_re_8_aw(void);
void m68k_op_and_re_8_al(void);
void m68k_op_and_re_16_ai(void);
void m68k_op_and_re_16_pi(void);
void m68k_op_and_re_16_pd(void);
void m68k_op_and_re_16_di(void);
void m68k_op_and_re_16_ix(void);
void m68k_op_and_re_16_aw(void);
void m68k_op_and_re_16_al(void);
void m68k_op_and_re_32_ai(void);
void m68k_op_and_re_32_pi(void);
void m68k_op_and_re_32_pd(void);
void m68k_op_and_re_32_di(void);
void m68k_op_and_re_32_ix(void);
void m68k_op_and_re_32_aw(void);
void m68k_op_and_re_32_al(void);
void m68k_op_andi_8_d(void);
void m68k_op_andi_8_ai(void);
void m68k_op_andi_8_pi(void);
void m68k_op_andi_8_pi7(void);
void m68k_op_andi_8_pd(void);
void m68k_op_andi_8_pd7(void);
void m68k_op_andi_8_di(void);
void m68k_op_andi_8_ix(void);
void m68k_op_andi_8_aw(void);
void m68k_op_andi_8_al(void);
void m68k_op_andi_16_d(void);
void m68k_op_andi_16_ai(void);
void m68k_op_andi_16_pi(void);
void m68k_op_andi_16_pd(void);
void m68k_op_andi_16_di(void);
void m68k_op_andi_16_ix(void);
void m68k_op_andi_16_aw(void);
void m68k_op_andi_16_al(void);
void m68k_op_andi_32_d(void);
void m68k_op_andi_32_ai(void);
void m68k_op_andi_32_pi(void);
void m68k_op_andi_32_pd(void);
void m68k_op_andi_32_di(void);
void m68k_op_andi_32_ix(void);
void m68k_op_andi_32_aw(void);
void m68k_op_andi_32_al(void);
void m68k_op_andi_to_ccr_16(void);
void m68k_op_andi_to_sr_16(void);
void m68k_op_asr_s_8(void);
void m68k_op_asr_s_16(void);
void m68k_op_asr_s_32(void);
void m68k_op_asr_r_8(void);
void m68k_op_asr_r_16(void);
void m68k_op_asr_r_32(void);
void m68k_op_asr_16_ai(void);
void m68k_op_asr_16_pi(void);
void m68k_op_asr_16_pd(void);
void m68k_op_asr_16_di(void);
void m68k_op_asr_16_ix(void);
void m68k_op_asr_16_aw(void);
void m68k_op_asr_16_al(void);
void m68k_op_asl_s_8(void);
void m68k_op_asl_s_16(void);
void m68k_op_asl_s_32(void);
void m68k_op_asl_r_8(void);
void m68k_op_asl_r_16(void);
void m68k_op_asl_r_32(void);
void m68k_op_asl_16_ai(void);
void m68k_op_asl_16_pi(void);
void m68k_op_asl_16_pd(void);
void m68k_op_asl_16_di(void);
void m68k_op_asl_16_ix(void);
void m68k_op_asl_16_aw(void);
void m68k_op_asl_16_al(void);
void m68k_op_bhi_8(void);
void m68k_op_bls_8(void);
void m68k_op_bcc_8(void);
void m68k_op_bcs_8(void);
void m68k_op_bne_8(void);
void m68k_op_beq_8(void);
void m68k_op_bvc_8(void);
void m68k_op_bvs_8(void);
void m68k_op_bpl_8(void);
void m68k_op_bmi_8(void);
void m68k_op_bge_8(void);
void m68k_op_blt_8(void);
void m68k_op_bgt_8(void);
void m68k_op_ble_8(void);
void m68k_op_bhi_16(void);
void m68k_op_bls_16(void);
void m68k_op_bcc_16(void);
void m68k_op_bcs_16(void);
void m68k_op_bne_16(void);
void m68k_op_beq_16(void);
void m68k_op_bvc_16(void);
void m68k_op_bvs_16(void);
void m68k_op_bpl_16(void);
void m68k_op_bmi_16(void);
void m68k_op_bge_16(void);
void m68k_op_blt_16(void);
void m68k_op_bgt_16(void);
void m68k_op_ble_16(void);
void m68k_op_bhi_32(void);
void m68k_op_bls_32(void);
void m68k_op_bcc_32(void);
void m68k_op_bcs_32(void);
void m68k_op_bne_32(void);
void m68k_op_beq_32(void);
void m68k_op_bvc_32(void);
void m68k_op_bvs_32(void);
void m68k_op_bpl_32(void);
void m68k_op_bmi_32(void);
void m68k_op_bge_32(void);
void m68k_op_blt_32(void);
void m68k_op_bgt_32(void);
void m68k_op_ble_32(void);
void m68k_op_bchg_r_32_d(void);
void m68k_op_bchg_r_8_ai(void);
void m68k_op_bchg_r_8_pi(void);
void m68k_op_bchg_r_8_pi7(void);
void m68k_op_bchg_r_8_pd(void);
void m68k_op_bchg_r_8_pd7(void);
void m68k_op_bchg_r_8_di(void);
void m68k_op_bchg_r_8_ix(void);
void m68k_op_bchg_r_8_aw(void);
void m68k_op_bchg_r_8_al(void);
void m68k_op_bchg_s_32_d(void);
void m68k_op_bchg_s_8_ai(void);
void m68k_op_bchg_s_8_pi(void);
void m68k_op_bchg_s_8_pi7(void);
void m68k_op_bchg_s_8_pd(void);
void m68k_op_bchg_s_8_pd7(void);
void m68k_op_bchg_s_8_di(void);
void m68k_op_bchg_s_8_ix(void);
void m68k_op_bchg_s_8_aw(void);
void m68k_op_bchg_s_8_al(void);
void m68k_op_bclr_r_32_d(void);
void m68k_op_bclr_r_8_ai(void);
void m68k_op_bclr_r_8_pi(void);
void m68k_op_bclr_r_8_pi7(void);
void m68k_op_bclr_r_8_pd(void);
void m68k_op_bclr_r_8_pd7(void);
void m68k_op_bclr_r_8_di(void);
void m68k_op_bclr_r_8_ix(void);
void m68k_op_bclr_r_8_aw(void);
void m68k_op_bclr_r_8_al(void);
void m68k_op_bclr_s_32_d(void);
void m68k_op_bclr_s_8_ai(void);
void m68k_op_bclr_s_8_pi(void);
void m68k_op_bclr_s_8_pi7(void);
void m68k_op_bclr_s_8_pd(void);
void m68k_op_bclr_s_8_pd7(void);
void m68k_op_bclr_s_8_di(void);
void m68k_op_bclr_s_8_ix(void);
void m68k_op_bclr_s_8_aw(void);
void m68k_op_bclr_s_8_al(void);
void m68k_op_bfchg_32_d(void);
void m68k_op_bfchg_32_ai(void);
void m68k_op_bfchg_32_di(void);
void m68k_op_bfchg_32_ix(void);
void m68k_op_bfchg_32_aw(void);
void m68k_op_bfchg_32_al(void);
void m68k_op_bfclr_32_d(void);
void m68k_op_bfclr_32_ai(void);
void m68k_op_bfclr_32_di(void);
void m68k_op_bfclr_32_ix(void);
void m68k_op_bfclr_32_aw(void);
void m68k_op_bfclr_32_al(void);
void m68k_op_bfexts_32_d(void);
void m68k_op_bfexts_32_ai(void);
void m68k_op_bfexts_32_di(void);
void m68k_op_bfexts_32_ix(void);
void m68k_op_bfexts_32_aw(void);
void m68k_op_bfexts_32_al(void);
void m68k_op_bfexts_32_pcdi(void);
void m68k_op_bfexts_32_pcix(void);
void m68k_op_bfextu_32_d(void);
void m68k_op_bfextu_32_ai(void);
void m68k_op_bfextu_32_di(void);
void m68k_op_bfextu_32_ix(void);
void m68k_op_bfextu_32_aw(void);
void m68k_op_bfextu_32_al(void);
void m68k_op_bfextu_32_pcdi(void);
void m68k_op_bfextu_32_pcix(void);
void m68k_op_bfffo_32_d(void);
void m68k_op_bfffo_32_ai(void);
void m68k_op_bfffo_32_di(void);
void m68k_op_bfffo_32_ix(void);
void m68k_op_bfffo_32_aw(void);
void m68k_op_bfffo_32_al(void);
void m68k_op_bfffo_32_pcdi(void);
void m68k_op_bfffo_32_pcix(void);
void m68k_op_bfins_32_d(void);
void m68k_op_bfins_32_ai(void);
void m68k_op_bfins_32_di(void);
void m68k_op_bfins_32_ix(void);
void m68k_op_bfins_32_aw(void);
void m68k_op_bfins_32_al(void);
void m68k_op_bfset_32_d(void);
void m68k_op_bfset_32_ai(void);
void m68k_op_bfset_32_di(void);
void m68k_op_bfset_32_ix(void);
void m68k_op_bfset_32_aw(void);
void m68k_op_bfset_32_al(void);
void m68k_op_bftst_32_d(void);
void m68k_op_bftst_32_ai(void);
void m68k_op_bftst_32_di(void);
void m68k_op_bftst_32_ix(void);
void m68k_op_bftst_32_aw(void);
void m68k_op_bftst_32_al(void);
void m68k_op_bftst_32_pcdi(void);
void m68k_op_bftst_32_pcix(void);
void m68k_op_bkpt(void);
void m68k_op_bra_8(void);
void m68k_op_bra_16(void);
void m68k_op_bra_32(void);
void m68k_op_bset_r_32_d(void);
void m68k_op_bset_r_8_ai(void);
void m68k_op_bset_r_8_pi(void);
void m68k_op_bset_r_8_pi7(void);
void m68k_op_bset_r_8_pd(void);
void m68k_op_bset_r_8_pd7(void);
void m68k_op_bset_r_8_di(void);
void m68k_op_bset_r_8_ix(void);
void m68k_op_bset_r_8_aw(void);
void m68k_op_bset_r_8_al(void);
void m68k_op_bset_s_32_d(void);
void m68k_op_bset_s_8_ai(void);
void m68k_op_bset_s_8_pi(void);
void m68k_op_bset_s_8_pi7(void);
void m68k_op_bset_s_8_pd(void);
void m68k_op_bset_s_8_pd7(void);
void m68k_op_bset_s_8_di(void);
void m68k_op_bset_s_8_ix(void);
void m68k_op_bset_s_8_aw(void);
void m68k_op_bset_s_8_al(void);
void m68k_op_bsr_8(void);
void m68k_op_bsr_16(void);
void m68k_op_bsr_32(void);
void m68k_op_btst_r_32_d(void);
void m68k_op_btst_r_8_ai(void);
void m68k_op_btst_r_8_pi(void);
void m68k_op_btst_r_8_pi7(void);
void m68k_op_btst_r_8_pd(void);
void m68k_op_btst_r_8_pd7(void);
void m68k_op_btst_r_8_di(void);
void m68k_op_btst_r_8_ix(void);
void m68k_op_btst_r_8_aw(void);
void m68k_op_btst_r_8_al(void);
void m68k_op_btst_r_8_pcdi(void);
void m68k_op_btst_r_8_pcix(void);
void m68k_op_btst_r_8_i(void);
void m68k_op_btst_s_32_d(void);
void m68k_op_btst_s_8_ai(void);
void m68k_op_btst_s_8_pi(void);
void m68k_op_btst_s_8_pi7(void);
void m68k_op_btst_s_8_pd(void);
void m68k_op_btst_s_8_pd7(void);
void m68k_op_btst_s_8_di(void);
void m68k_op_btst_s_8_ix(void);
void m68k_op_btst_s_8_aw(void);
void m68k_op_btst_s_8_al(void);
void m68k_op_btst_s_8_pcdi(void);
void m68k_op_btst_s_8_pcix(void);
void m68k_op_callm_32_ai(void);
void m68k_op_callm_32_di(void);
void m68k_op_callm_32_ix(void);
void m68k_op_callm_32_aw(void);
void m68k_op_callm_32_al(void);
void m68k_op_callm_32_pcdi(void);
void m68k_op_callm_32_pcix(void);
void m68k_op_cas_8_ai(void);
void m68k_op_cas_8_pi(void);
void m68k_op_cas_8_pi7(void);
void m68k_op_cas_8_pd(void);
void m68k_op_cas_8_pd7(void);
void m68k_op_cas_8_di(void);
void m68k_op_cas_8_ix(void);
void m68k_op_cas_8_aw(void);
void m68k_op_cas_8_al(void);
void m68k_op_cas_16_ai(void);
void m68k_op_cas_16_pi(void);
void m68k_op_cas_16_pd(void);
void m68k_op_cas_16_di(void);
void m68k_op_cas_16_ix(void);
void m68k_op_cas_16_aw(void);
void m68k_op_cas_16_al(void);
void m68k_op_cas_32_ai(void);
void m68k_op_cas_32_pi(void);
void m68k_op_cas_32_pd(void);
void m68k_op_cas_32_di(void);
void m68k_op_cas_32_ix(void);
void m68k_op_cas_32_aw(void);
void m68k_op_cas_32_al(void);
void m68k_op_cas2_16(void);
void m68k_op_cas2_32(void);
void m68k_op_chk_16_d(void);
void m68k_op_chk_16_ai(void);
void m68k_op_chk_16_pi(void);
void m68k_op_chk_16_pd(void);
void m68k_op_chk_16_di(void);
void m68k_op_chk_16_ix(void);
void m68k_op_chk_16_aw(void);
void m68k_op_chk_16_al(void);
void m68k_op_chk_16_pcdi(void);
void m68k_op_chk_16_pcix(void);
void m68k_op_chk_16_i(void);
void m68k_op_chk_32_d(void);
void m68k_op_chk_32_ai(void);
void m68k_op_chk_32_pi(void);
void m68k_op_chk_32_pd(void);
void m68k_op_chk_32_di(void);
void m68k_op_chk_32_ix(void);
void m68k_op_chk_32_aw(void);
void m68k_op_chk_32_al(void);
void m68k_op_chk_32_pcdi(void);
void m68k_op_chk_32_pcix(void);
void m68k_op_chk_32_i(void);
void m68k_op_chk2_cmp2_8_ai(void);
void m68k_op_chk2_cmp2_8_di(void);
void m68k_op_chk2_cmp2_8_ix(void);
void m68k_op_chk2_cmp2_8_aw(void);
void m68k_op_chk2_cmp2_8_al(void);
void m68k_op_chk2_cmp2_8_pcdi(void);
void m68k_op_chk2_cmp2_8_pcix(void);
void m68k_op_chk2_cmp2_16_ai(void);
void m68k_op_chk2_cmp2_16_di(void);
void m68k_op_chk2_cmp2_16_ix(void);
void m68k_op_chk2_cmp2_16_aw(void);
void m68k_op_chk2_cmp2_16_al(void);
void m68k_op_chk2_cmp2_16_pcdi(void);
void m68k_op_chk2_cmp2_16_pcix(void);
void m68k_op_chk2_cmp2_32_ai(void);
void m68k_op_chk2_cmp2_32_di(void);
void m68k_op_chk2_cmp2_32_ix(void);
void m68k_op_chk2_cmp2_32_aw(void);
void m68k_op_chk2_cmp2_32_al(void);
void m68k_op_chk2_cmp2_32_pcdi(void);
void m68k_op_chk2_cmp2_32_pcix(void);
void m68k_op_clr_8_d(void);
void m68k_op_clr_8_ai(void);
void m68k_op_clr_8_pi(void);
void m68k_op_clr_8_pi7(void);
void m68k_op_clr_8_pd(void);
void m68k_op_clr_8_pd7(void);
void m68k_op_clr_8_di(void);
void m68k_op_clr_8_ix(void);
void m68k_op_clr_8_aw(void);
void m68k_op_clr_8_al(void);
void m68k_op_clr_16_d(void);
void m68k_op_clr_16_ai(void);
void m68k_op_clr_16_pi(void);
void m68k_op_clr_16_pd(void);
void m68k_op_clr_16_di(void);
void m68k_op_clr_16_ix(void);
void m68k_op_clr_16_aw(void);
void m68k_op_clr_16_al(void);
void m68k_op_clr_32_d(void);
void m68k_op_clr_32_ai(void);
void m68k_op_clr_32_pi(void);
void m68k_op_clr_32_pd(void);
void m68k_op_clr_32_di(void);
void m68k_op_clr_32_ix(void);
void m68k_op_clr_32_aw(void);
void m68k_op_clr_32_al(void);
void m68k_op_cmp_8_d(void);
void m68k_op_cmp_8_ai(void);
void m68k_op_cmp_8_pi(void);
void m68k_op_cmp_8_pi7(void);
void m68k_op_cmp_8_pd(void);
void m68k_op_cmp_8_pd7(void);
void m68k_op_cmp_8_di(void);
void m68k_op_cmp_8_ix(void);
void m68k_op_cmp_8_aw(void);
void m68k_op_cmp_8_al(void);
void m68k_op_cmp_8_pcdi(void);
void m68k_op_cmp_8_pcix(void);
void m68k_op_cmp_8_i(void);
void m68k_op_cmp_16_d(void);
void m68k_op_cmp_16_a(void);
void m68k_op_cmp_16_ai(void);
void m68k_op_cmp_16_pi(void);
void m68k_op_cmp_16_pd(void);
void m68k_op_cmp_16_di(void);
void m68k_op_cmp_16_ix(void);
void m68k_op_cmp_16_aw(void);
void m68k_op_cmp_16_al(void);
void m68k_op_cmp_16_pcdi(void);
void m68k_op_cmp_16_pcix(void);
void m68k_op_cmp_16_i(void);
void m68k_op_cmp_32_d(void);
void m68k_op_cmp_32_a(void);
void m68k_op_cmp_32_ai(void);
void m68k_op_cmp_32_pi(void);
void m68k_op_cmp_32_pd(void);
void m68k_op_cmp_32_di(void);
void m68k_op_cmp_32_ix(void);
void m68k_op_cmp_32_aw(void);
void m68k_op_cmp_32_al(void);
void m68k_op_cmp_32_pcdi(void);
void m68k_op_cmp_32_pcix(void);
void m68k_op_cmp_32_i(void);
void m68k_op_cmpa_16_d(void);
void m68k_op_cmpa_16_a(void);
void m68k_op_cmpa_16_ai(void);
void m68k_op_cmpa_16_pi(void);
void m68k_op_cmpa_16_pd(void);
void m68k_op_cmpa_16_di(void);
void m68k_op_cmpa_16_ix(void);
void m68k_op_cmpa_16_aw(void);
void m68k_op_cmpa_16_al(void);
void m68k_op_cmpa_16_pcdi(void);
void m68k_op_cmpa_16_pcix(void);
void m68k_op_cmpa_16_i(void);
void m68k_op_cmpa_32_d(void);
void m68k_op_cmpa_32_a(void);
void m68k_op_cmpa_32_ai(void);
void m68k_op_cmpa_32_pi(void);
void m68k_op_cmpa_32_pd(void);
void m68k_op_cmpa_32_di(void);
void m68k_op_cmpa_32_ix(void);
void m68k_op_cmpa_32_aw(void);
void m68k_op_cmpa_32_al(void);
void m68k_op_cmpa_32_pcdi(void);
void m68k_op_cmpa_32_pcix(void);
void m68k_op_cmpa_32_i(void);
void m68k_op_cmpi_8_d(void);
void m68k_op_cmpi_8_ai(void);
void m68k_op_cmpi_8_pi(void);
void m68k_op_cmpi_8_pi7(void);
void m68k_op_cmpi_8_pd(void);
void m68k_op_cmpi_8_pd7(void);
void m68k_op_cmpi_8_di(void);
void m68k_op_cmpi_8_ix(void);
void m68k_op_cmpi_8_aw(void);
void m68k_op_cmpi_8_al(void);
void m68k_op_cmpi_8_pcdi(void);
void m68k_op_cmpi_8_pcix(void);
void m68k_op_cmpi_16_d(void);
void m68k_op_cmpi_16_ai(void);
void m68k_op_cmpi_16_pi(void);
void m68k_op_cmpi_16_pd(void);
void m68k_op_cmpi_16_di(void);
void m68k_op_cmpi_16_ix(void);
void m68k_op_cmpi_16_aw(void);
void m68k_op_cmpi_16_al(void);
void m68k_op_cmpi_16_pcdi(void);
void m68k_op_cmpi_16_pcix(void);
void m68k_op_cmpi_32_d(void);
void m68k_op_cmpi_32_ai(void);
void m68k_op_cmpi_32_pi(void);
void m68k_op_cmpi_32_pd(void);
void m68k_op_cmpi_32_di(void);
void m68k_op_cmpi_32_ix(void);
void m68k_op_cmpi_32_aw(void);
void m68k_op_cmpi_32_al(void);
void m68k_op_cmpi_32_pcdi(void);
void m68k_op_cmpi_32_pcix(void);
void m68k_op_cmpm_8_ax7(void);
void m68k_op_cmpm_8_ay7(void);
void m68k_op_cmpm_8_axy7(void);
void m68k_op_cmpm_8(void);
void m68k_op_cmpm_16(void);
void m68k_op_cmpm_32(void);
void m68k_op_cpbcc_32(void);
void m68k_op_cpdbcc_32(void);
void m68k_op_cpgen_32(void);
void m68k_op_cpscc_32(void);
void m68k_op_cptrapcc_32(void);
void m68k_op_dbt_16(void);
void m68k_op_dbf_16(void);
void m68k_op_dbhi_16(void);
void m68k_op_dbls_16(void);
void m68k_op_dbcc_16(void);
void m68k_op_dbcs_16(void);
void m68k_op_dbne_16(void);
void m68k_op_dbeq_16(void);
void m68k_op_dbvc_16(void);
void m68k_op_dbvs_16(void);
void m68k_op_dbpl_16(void);
void m68k_op_dbmi_16(void);
void m68k_op_dbge_16(void);
void m68k_op_dblt_16(void);
void m68k_op_dbgt_16(void);
void m68k_op_dble_16(void);
void m68k_op_divs_16_d(void);
void m68k_op_divs_16_ai(void);
void m68k_op_divs_16_pi(void);
void m68k_op_divs_16_pd(void);
void m68k_op_divs_16_di(void);
void m68k_op_divs_16_ix(void);
void m68k_op_divs_16_aw(void);
void m68k_op_divs_16_al(void);
void m68k_op_divs_16_pcdi(void);
void m68k_op_divs_16_pcix(void);
void m68k_op_divs_16_i(void);
void m68k_op_divu_16_d(void);
void m68k_op_divu_16_ai(void);
void m68k_op_divu_16_pi(void);
void m68k_op_divu_16_pd(void);
void m68k_op_divu_16_di(void);
void m68k_op_divu_16_ix(void);
void m68k_op_divu_16_aw(void);
void m68k_op_divu_16_al(void);
void m68k_op_divu_16_pcdi(void);
void m68k_op_divu_16_pcix(void);
void m68k_op_divu_16_i(void);
void m68k_op_divl_32_d(void);
void m68k_op_divl_32_ai(void);
void m68k_op_divl_32_pi(void);
void m68k_op_divl_32_pd(void);
void m68k_op_divl_32_di(void);
void m68k_op_divl_32_ix(void);
void m68k_op_divl_32_aw(void);
void m68k_op_divl_32_al(void);
void m68k_op_divl_32_pcdi(void);
void m68k_op_divl_32_pcix(void);
void m68k_op_divl_32_i(void);
void m68k_op_eor_8_d(void);
void m68k_op_eor_8_ai(void);
void m68k_op_eor_8_pi(void);
void m68k_op_eor_8_pi7(void);
void m68k_op_eor_8_pd(void);
void m68k_op_eor_8_pd7(void);
void m68k_op_eor_8_di(void);
void m68k_op_eor_8_ix(void);
void m68k_op_eor_8_aw(void);
void m68k_op_eor_8_al(void);
void m68k_op_eor_16_d(void);
void m68k_op_eor_16_ai(void);
void m68k_op_eor_16_pi(void);
void m68k_op_eor_16_pd(void);
void m68k_op_eor_16_di(void);
void m68k_op_eor_16_ix(void);
void m68k_op_eor_16_aw(void);
void m68k_op_eor_16_al(void);
void m68k_op_eor_32_d(void);
void m68k_op_eor_32_ai(void);
void m68k_op_eor_32_pi(void);
void m68k_op_eor_32_pd(void);
void m68k_op_eor_32_di(void);
void m68k_op_eor_32_ix(void);
void m68k_op_eor_32_aw(void);
void m68k_op_eor_32_al(void);
void m68k_op_eori_8_d(void);
void m68k_op_eori_8_ai(void);
void m68k_op_eori_8_pi(void);
void m68k_op_eori_8_pi7(void);
void m68k_op_eori_8_pd(void);
void m68k_op_eori_8_pd7(void);
void m68k_op_eori_8_di(void);
void m68k_op_eori_8_ix(void);
void m68k_op_eori_8_aw(void);
void m68k_op_eori_8_al(void);
void m68k_op_eori_16_d(void);
void m68k_op_eori_16_ai(void);
void m68k_op_eori_16_pi(void);
void m68k_op_eori_16_pd(void);
void m68k_op_eori_16_di(void);
void m68k_op_eori_16_ix(void);
void m68k_op_eori_16_aw(void);
void m68k_op_eori_16_al(void);
void m68k_op_eori_32_d(void);
void m68k_op_eori_32_ai(void);
void m68k_op_eori_32_pi(void);
void m68k_op_eori_32_pd(void);
void m68k_op_eori_32_di(void);
void m68k_op_eori_32_ix(void);
void m68k_op_eori_32_aw(void);
void m68k_op_eori_32_al(void);
void m68k_op_eori_to_ccr_16(void);
void m68k_op_eori_to_sr_16(void);
void m68k_op_exg_dd_32(void);
void m68k_op_exg_aa_32(void);
void m68k_op_exg_da_32(void);
void m68k_op_ext_16(void);
void m68k_op_ext_32(void);
void m68k_op_extb_32(void);
void m68k_op_illegal(void);
void m68k_op_jmp_32_ai(void);
void m68k_op_jmp_32_di(void);
void m68k_op_jmp_32_ix(void);
void m68k_op_jmp_32_aw(void);
void m68k_op_jmp_32_al(void);
void m68k_op_jmp_32_pcdi(void);
void m68k_op_jmp_32_pcix(void);
void m68k_op_jsr_32_ai(void);
void m68k_op_jsr_32_di(void);
void m68k_op_jsr_32_ix(void);
void m68k_op_jsr_32_aw(void);
void m68k_op_jsr_32_al(void);
void m68k_op_jsr_32_pcdi(void);
void m68k_op_jsr_32_pcix(void);
void m68k_op_lea_32_ai(void);
void m68k_op_lea_32_di(void);
void m68k_op_lea_32_ix(void);
void m68k_op_lea_32_aw(void);
void m68k_op_lea_32_al(void);
void m68k_op_lea_32_pcdi(void);
void m68k_op_lea_32_pcix(void);
void m68k_op_link_16_a7(void);
void m68k_op_link_16(void);
void m68k_op_link_32_a7(void);
void m68k_op_link_32(void);
void m68k_op_lsr_s_8(void);
void m68k_op_lsr_s_16(void);
void m68k_op_lsr_s_32(void);
void m68k_op_lsr_r_8(void);
void m68k_op_lsr_r_16(void);
void m68k_op_lsr_r_32(void);
void m68k_op_lsr_16_ai(void);
void m68k_op_lsr_16_pi(void);
void m68k_op_lsr_16_pd(void);
void m68k_op_lsr_16_di(void);
void m68k_op_lsr_16_ix(void);
void m68k_op_lsr_16_aw(void);
void m68k_op_lsr_16_al(void);
void m68k_op_lsl_s_8(void);
void m68k_op_lsl_s_16(void);
void m68k_op_lsl_s_32(void);
void m68k_op_lsl_r_8(void);
void m68k_op_lsl_r_16(void);
void m68k_op_lsl_r_32(void);
void m68k_op_lsl_16_ai(void);
void m68k_op_lsl_16_pi(void);
void m68k_op_lsl_16_pd(void);
void m68k_op_lsl_16_di(void);
void m68k_op_lsl_16_ix(void);
void m68k_op_lsl_16_aw(void);
void m68k_op_lsl_16_al(void);
void m68k_op_move_dd_8_d(void);
void m68k_op_move_dd_8_ai(void);
void m68k_op_move_dd_8_pi(void);
void m68k_op_move_dd_8_pi7(void);
void m68k_op_move_dd_8_pd(void);
void m68k_op_move_dd_8_pd7(void);
void m68k_op_move_dd_8_di(void);
void m68k_op_move_dd_8_ix(void);
void m68k_op_move_dd_8_aw(void);
void m68k_op_move_dd_8_al(void);
void m68k_op_move_dd_8_pcdi(void);
void m68k_op_move_dd_8_pcix(void);
void m68k_op_move_dd_8_i(void);
void m68k_op_move_ai_8_d(void);
void m68k_op_move_ai_8_ai(void);
void m68k_op_move_ai_8_pi(void);
void m68k_op_move_ai_8_pi7(void);
void m68k_op_move_ai_8_pd(void);
void m68k_op_move_ai_8_pd7(void);
void m68k_op_move_ai_8_di(void);
void m68k_op_move_ai_8_ix(void);
void m68k_op_move_ai_8_aw(void);
void m68k_op_move_ai_8_al(void);
void m68k_op_move_ai_8_pcdi(void);
void m68k_op_move_ai_8_pcix(void);
void m68k_op_move_ai_8_i(void);
void m68k_op_move_pi7_8_d(void);
void m68k_op_move_pi_8_d(void);
void m68k_op_move_pi7_8_ai(void);
void m68k_op_move_pi7_8_pi(void);
void m68k_op_move_pi7_8_pi7(void);
void m68k_op_move_pi7_8_pd(void);
void m68k_op_move_pi7_8_pd7(void);
void m68k_op_move_pi7_8_di(void);
void m68k_op_move_pi7_8_ix(void);
void m68k_op_move_pi7_8_aw(void);
void m68k_op_move_pi7_8_al(void);
void m68k_op_move_pi7_8_pcdi(void);
void m68k_op_move_pi7_8_pcix(void);
void m68k_op_move_pi7_8_i(void);
void m68k_op_move_pi_8_ai(void);
void m68k_op_move_pi_8_pi(void);
void m68k_op_move_pi_8_pi7(void);
void m68k_op_move_pi_8_pd(void);
void m68k_op_move_pi_8_pd7(void);
void m68k_op_move_pi_8_di(void);
void m68k_op_move_pi_8_ix(void);
void m68k_op_move_pi_8_aw(void);
void m68k_op_move_pi_8_al(void);
void m68k_op_move_pi_8_pcdi(void);
void m68k_op_move_pi_8_pcix(void);
void m68k_op_move_pi_8_i(void);
void m68k_op_move_pd7_8_d(void);
void m68k_op_move_pd_8_d(void);
void m68k_op_move_pd7_8_ai(void);
void m68k_op_move_pd7_8_pi(void);
void m68k_op_move_pd7_8_pi7(void);
void m68k_op_move_pd7_8_pd(void);
void m68k_op_move_pd7_8_pd7(void);
void m68k_op_move_pd7_8_di(void);
void m68k_op_move_pd7_8_ix(void);
void m68k_op_move_pd7_8_aw(void);
void m68k_op_move_pd7_8_al(void);
void m68k_op_move_pd7_8_pcdi(void);
void m68k_op_move_pd7_8_pcix(void);
void m68k_op_move_pd7_8_i(void);
void m68k_op_move_pd_8_ai(void);
void m68k_op_move_pd_8_pi(void);
void m68k_op_move_pd_8_pi7(void);
void m68k_op_move_pd_8_pd(void);
void m68k_op_move_pd_8_pd7(void);
void m68k_op_move_pd_8_di(void);
void m68k_op_move_pd_8_ix(void);
void m68k_op_move_pd_8_aw(void);
void m68k_op_move_pd_8_al(void);
void m68k_op_move_pd_8_pcdi(void);
void m68k_op_move_pd_8_pcix(void);
void m68k_op_move_pd_8_i(void);
void m68k_op_move_di_8_d(void);
void m68k_op_move_di_8_ai(void);
void m68k_op_move_di_8_pi(void);
void m68k_op_move_di_8_pi7(void);
void m68k_op_move_di_8_pd(void);
void m68k_op_move_di_8_pd7(void);
void m68k_op_move_di_8_di(void);
void m68k_op_move_di_8_ix(void);
void m68k_op_move_di_8_aw(void);
void m68k_op_move_di_8_al(void);
void m68k_op_move_di_8_pcdi(void);
void m68k_op_move_di_8_pcix(void);
void m68k_op_move_di_8_i(void);
void m68k_op_move_ix_8_d(void);
void m68k_op_move_ix_8_ai(void);
void m68k_op_move_ix_8_pi(void);
void m68k_op_move_ix_8_pi7(void);
void m68k_op_move_ix_8_pd(void);
void m68k_op_move_ix_8_pd7(void);
void m68k_op_move_ix_8_di(void);
void m68k_op_move_ix_8_ix(void);
void m68k_op_move_ix_8_aw(void);
void m68k_op_move_ix_8_al(void);
void m68k_op_move_ix_8_pcdi(void);
void m68k_op_move_ix_8_pcix(void);
void m68k_op_move_ix_8_i(void);
void m68k_op_move_aw_8_d(void);
void m68k_op_move_aw_8_ai(void);
void m68k_op_move_aw_8_pi(void);
void m68k_op_move_aw_8_pi7(void);
void m68k_op_move_aw_8_pd(void);
void m68k_op_move_aw_8_pd7(void);
void m68k_op_move_aw_8_di(void);
void m68k_op_move_aw_8_ix(void);
void m68k_op_move_aw_8_aw(void);
void m68k_op_move_aw_8_al(void);
void m68k_op_move_aw_8_pcdi(void);
void m68k_op_move_aw_8_pcix(void);
void m68k_op_move_aw_8_i(void);
void m68k_op_move_al_8_d(void);
void m68k_op_move_al_8_ai(void);
void m68k_op_move_al_8_pi(void);
void m68k_op_move_al_8_pi7(void);
void m68k_op_move_al_8_pd(void);
void m68k_op_move_al_8_pd7(void);
void m68k_op_move_al_8_di(void);
void m68k_op_move_al_8_ix(void);
void m68k_op_move_al_8_aw(void);
void m68k_op_move_al_8_al(void);
void m68k_op_move_al_8_pcdi(void);
void m68k_op_move_al_8_pcix(void);
void m68k_op_move_al_8_i(void);
void m68k_op_move_dd_16_d(void);
void m68k_op_move_dd_16_a(void);
void m68k_op_move_dd_16_ai(void);
void m68k_op_move_dd_16_pi(void);
void m68k_op_move_dd_16_pd(void);
void m68k_op_move_dd_16_di(void);
void m68k_op_move_dd_16_ix(void);
void m68k_op_move_dd_16_aw(void);
void m68k_op_move_dd_16_al(void);
void m68k_op_move_dd_16_pcdi(void);
void m68k_op_move_dd_16_pcix(void);
void m68k_op_move_dd_16_i(void);
void m68k_op_move_ai_16_d(void);
void m68k_op_move_ai_16_a(void);
void m68k_op_move_ai_16_ai(void);
void m68k_op_move_ai_16_pi(void);
void m68k_op_move_ai_16_pd(void);
void m68k_op_move_ai_16_di(void);
void m68k_op_move_ai_16_ix(void);
void m68k_op_move_ai_16_aw(void);
void m68k_op_move_ai_16_al(void);
void m68k_op_move_ai_16_pcdi(void);
void m68k_op_move_ai_16_pcix(void);
void m68k_op_move_ai_16_i(void);
void m68k_op_move_pi_16_d(void);
void m68k_op_move_pi_16_a(void);
void m68k_op_move_pi_16_ai(void);
void m68k_op_move_pi_16_pi(void);
void m68k_op_move_pi_16_pd(void);
void m68k_op_move_pi_16_di(void);
void m68k_op_move_pi_16_ix(void);
void m68k_op_move_pi_16_aw(void);
void m68k_op_move_pi_16_al(void);
void m68k_op_move_pi_16_pcdi(void);
void m68k_op_move_pi_16_pcix(void);
void m68k_op_move_pi_16_i(void);
void m68k_op_move_pd_16_d(void);
void m68k_op_move_pd_16_a(void);
void m68k_op_move_pd_16_ai(void);
void m68k_op_move_pd_16_pi(void);
void m68k_op_move_pd_16_pd(void);
void m68k_op_move_pd_16_di(void);
void m68k_op_move_pd_16_ix(void);
void m68k_op_move_pd_16_aw(void);
void m68k_op_move_pd_16_al(void);
void m68k_op_move_pd_16_pcdi(void);
void m68k_op_move_pd_16_pcix(void);
void m68k_op_move_pd_16_i(void);
void m68k_op_move_di_16_d(void);
void m68k_op_move_di_16_a(void);
void m68k_op_move_di_16_ai(void);
void m68k_op_move_di_16_pi(void);
void m68k_op_move_di_16_pd(void);
void m68k_op_move_di_16_di(void);
void m68k_op_move_di_16_ix(void);
void m68k_op_move_di_16_aw(void);
void m68k_op_move_di_16_al(void);
void m68k_op_move_di_16_pcdi(void);
void m68k_op_move_di_16_pcix(void);
void m68k_op_move_di_16_i(void);
void m68k_op_move_ix_16_d(void);
void m68k_op_move_ix_16_a(void);
void m68k_op_move_ix_16_ai(void);
void m68k_op_move_ix_16_pi(void);
void m68k_op_move_ix_16_pd(void);
void m68k_op_move_ix_16_di(void);
void m68k_op_move_ix_16_ix(void);
void m68k_op_move_ix_16_aw(void);
void m68k_op_move_ix_16_al(void);
void m68k_op_move_ix_16_pcdi(void);
void m68k_op_move_ix_16_pcix(void);
void m68k_op_move_ix_16_i(void);
void m68k_op_move_aw_16_d(void);
void m68k_op_move_aw_16_a(void);
void m68k_op_move_aw_16_ai(void);
void m68k_op_move_aw_16_pi(void);
void m68k_op_move_aw_16_pd(void);
void m68k_op_move_aw_16_di(void);
void m68k_op_move_aw_16_ix(void);
void m68k_op_move_aw_16_aw(void);
void m68k_op_move_aw_16_al(void);
void m68k_op_move_aw_16_pcdi(void);
void m68k_op_move_aw_16_pcix(void);
void m68k_op_move_aw_16_i(void);
void m68k_op_move_al_16_d(void);
void m68k_op_move_al_16_a(void);
void m68k_op_move_al_16_ai(void);
void m68k_op_move_al_16_pi(void);
void m68k_op_move_al_16_pd(void);
void m68k_op_move_al_16_di(void);
void m68k_op_move_al_16_ix(void);
void m68k_op_move_al_16_aw(void);
void m68k_op_move_al_16_al(void);
void m68k_op_move_al_16_pcdi(void);
void m68k_op_move_al_16_pcix(void);
void m68k_op_move_al_16_i(void);
void m68k_op_move_dd_32_d(void);
void m68k_op_move_dd_32_a(void);
void m68k_op_move_dd_32_ai(void);
void m68k_op_move_dd_32_pi(void);
void m68k_op_move_dd_32_pd(void);
void m68k_op_move_dd_32_di(void);
void m68k_op_move_dd_32_ix(void);
void m68k_op_move_dd_32_aw(void);
void m68k_op_move_dd_32_al(void);
void m68k_op_move_dd_32_pcdi(void);
void m68k_op_move_dd_32_pcix(void);
void m68k_op_move_dd_32_i(void);
void m68k_op_move_ai_32_d(void);
void m68k_op_move_ai_32_a(void);
void m68k_op_move_ai_32_ai(void);
void m68k_op_move_ai_32_pi(void);
void m68k_op_move_ai_32_pd(void);
void m68k_op_move_ai_32_di(void);
void m68k_op_move_ai_32_ix(void);
void m68k_op_move_ai_32_aw(void);
void m68k_op_move_ai_32_al(void);
void m68k_op_move_ai_32_pcdi(void);
void m68k_op_move_ai_32_pcix(void);
void m68k_op_move_ai_32_i(void);
void m68k_op_move_pi_32_d(void);
void m68k_op_move_pi_32_a(void);
void m68k_op_move_pi_32_ai(void);
void m68k_op_move_pi_32_pi(void);
void m68k_op_move_pi_32_pd(void);
void m68k_op_move_pi_32_di(void);
void m68k_op_move_pi_32_ix(void);
void m68k_op_move_pi_32_aw(void);
void m68k_op_move_pi_32_al(void);
void m68k_op_move_pi_32_pcdi(void);
void m68k_op_move_pi_32_pcix(void);
void m68k_op_move_pi_32_i(void);
void m68k_op_move_pd_32_d(void);
void m68k_op_move_pd_32_a(void);
void m68k_op_move_pd_32_ai(void);
void m68k_op_move_pd_32_pi(void);
void m68k_op_move_pd_32_pd(void);
void m68k_op_move_pd_32_di(void);
void m68k_op_move_pd_32_ix(void);
void m68k_op_move_pd_32_aw(void);
void m68k_op_move_pd_32_al(void);
void m68k_op_move_pd_32_pcdi(void);
void m68k_op_move_pd_32_pcix(void);
void m68k_op_move_pd_32_i(void);
void m68k_op_move_di_32_d(void);
void m68k_op_move_di_32_a(void);
void m68k_op_move_di_32_ai(void);
void m68k_op_move_di_32_pi(void);
void m68k_op_move_di_32_pd(void);
void m68k_op_move_di_32_di(void);
void m68k_op_move_di_32_ix(void);
void m68k_op_move_di_32_aw(void);
void m68k_op_move_di_32_al(void);
void m68k_op_move_di_32_pcdi(void);
void m68k_op_move_di_32_pcix(void);
void m68k_op_move_di_32_i(void);
void m68k_op_move_ix_32_d(void);
void m68k_op_move_ix_32_a(void);
void m68k_op_move_ix_32_ai(void);
void m68k_op_move_ix_32_pi(void);
void m68k_op_move_ix_32_pd(void);
void m68k_op_move_ix_32_di(void);
void m68k_op_move_ix_32_ix(void);
void m68k_op_move_ix_32_aw(void);
void m68k_op_move_ix_32_al(void);
void m68k_op_move_ix_32_pcdi(void);
void m68k_op_move_ix_32_pcix(void);
void m68k_op_move_ix_32_i(void);
void m68k_op_move_aw_32_d(void);
void m68k_op_move_aw_32_a(void);
void m68k_op_move_aw_32_ai(void);
void m68k_op_move_aw_32_pi(void);
void m68k_op_move_aw_32_pd(void);
void m68k_op_move_aw_32_di(void);
void m68k_op_move_aw_32_ix(void);
void m68k_op_move_aw_32_aw(void);
void m68k_op_move_aw_32_al(void);
void m68k_op_move_aw_32_pcdi(void);
void m68k_op_move_aw_32_pcix(void);
void m68k_op_move_aw_32_i(void);
void m68k_op_move_al_32_d(void);
void m68k_op_move_al_32_a(void);
void m68k_op_move_al_32_ai(void);
void m68k_op_move_al_32_pi(void);
void m68k_op_move_al_32_pd(void);
void m68k_op_move_al_32_di(void);
void m68k_op_move_al_32_ix(void);
void m68k_op_move_al_32_aw(void);
void m68k_op_move_al_32_al(void);
void m68k_op_move_al_32_pcdi(void);
void m68k_op_move_al_32_pcix(void);
void m68k_op_move_al_32_i(void);
void m68k_op_movea_16_d(void);
void m68k_op_movea_16_a(void);
void m68k_op_movea_16_ai(void);
void m68k_op_movea_16_pi(void);
void m68k_op_movea_16_pd(void);
void m68k_op_movea_16_di(void);
void m68k_op_movea_16_ix(void);
void m68k_op_movea_16_aw(void);
void m68k_op_movea_16_al(void);
void m68k_op_movea_16_pcdi(void);
void m68k_op_movea_16_pcix(void);
void m68k_op_movea_16_i(void);
void m68k_op_movea_32_d(void);
void m68k_op_movea_32_a(void);
void m68k_op_movea_32_ai(void);
void m68k_op_movea_32_pi(void);
void m68k_op_movea_32_pd(void);
void m68k_op_movea_32_di(void);
void m68k_op_movea_32_ix(void);
void m68k_op_movea_32_aw(void);
void m68k_op_movea_32_al(void);
void m68k_op_movea_32_pcdi(void);
void m68k_op_movea_32_pcix(void);
void m68k_op_movea_32_i(void);
void m68k_op_move_fr_ccr_16_d(void);
void m68k_op_move_fr_ccr_16_ai(void);
void m68k_op_move_fr_ccr_16_pi(void);
void m68k_op_move_fr_ccr_16_pd(void);
void m68k_op_move_fr_ccr_16_di(void);
void m68k_op_move_fr_ccr_16_ix(void);
void m68k_op_move_fr_ccr_16_aw(void);
void m68k_op_move_fr_ccr_16_al(void);
void m68k_op_move_to_ccr_16_d(void);
void m68k_op_move_to_ccr_16_ai(void);
void m68k_op_move_to_ccr_16_pi(void);
void m68k_op_move_to_ccr_16_pd(void);
void m68k_op_move_to_ccr_16_di(void);
void m68k_op_move_to_ccr_16_ix(void);
void m68k_op_move_to_ccr_16_aw(void);
void m68k_op_move_to_ccr_16_al(void);
void m68k_op_move_to_ccr_16_pcdi(void);
void m68k_op_move_to_ccr_16_pcix(void);
void m68k_op_move_to_ccr_16_i(void);
void m68k_op_move_fr_sr_16_d(void);
void m68k_op_move_fr_sr_16_ai(void);
void m68k_op_move_fr_sr_16_pi(void);
void m68k_op_move_fr_sr_16_pd(void);
void m68k_op_move_fr_sr_16_di(void);
void m68k_op_move_fr_sr_16_ix(void);
void m68k_op_move_fr_sr_16_aw(void);
void m68k_op_move_fr_sr_16_al(void);
void m68k_op_move_to_sr_16_d(void);
void m68k_op_move_to_sr_16_ai(void);
void m68k_op_move_to_sr_16_pi(void);
void m68k_op_move_to_sr_16_pd(void);
void m68k_op_move_to_sr_16_di(void);
void m68k_op_move_to_sr_16_ix(void);
void m68k_op_move_to_sr_16_aw(void);
void m68k_op_move_to_sr_16_al(void);
void m68k_op_move_to_sr_16_pcdi(void);
void m68k_op_move_to_sr_16_pcix(void);
void m68k_op_move_to_sr_16_i(void);
void m68k_op_move_fr_usp_32(void);
void m68k_op_move_to_usp_32(void);
void m68k_op_movec_cr_32(void);
void m68k_op_movec_rc_32(void);
void m68k_op_movem_re_16_pd(void);
void m68k_op_movem_re_16_ai(void);
void m68k_op_movem_re_16_di(void);
void m68k_op_movem_re_16_ix(void);
void m68k_op_movem_re_16_aw(void);
void m68k_op_movem_re_16_al(void);
void m68k_op_movem_re_32_pd(void);
void m68k_op_movem_re_32_ai(void);
void m68k_op_movem_re_32_di(void);
void m68k_op_movem_re_32_ix(void);
void m68k_op_movem_re_32_aw(void);
void m68k_op_movem_re_32_al(void);
void m68k_op_movem_er_16_pi(void);
void m68k_op_movem_er_16_ai(void);
void m68k_op_movem_er_16_di(void);
void m68k_op_movem_er_16_ix(void);
void m68k_op_movem_er_16_aw(void);
void m68k_op_movem_er_16_al(void);
void m68k_op_movem_er_16_pcdi(void);
void m68k_op_movem_er_16_pcix(void);
void m68k_op_movem_er_32_pi(void);
void m68k_op_movem_er_32_ai(void);
void m68k_op_movem_er_32_di(void);
void m68k_op_movem_er_32_ix(void);
void m68k_op_movem_er_32_aw(void);
void m68k_op_movem_er_32_al(void);
void m68k_op_movem_er_32_pcdi(void);
void m68k_op_movem_er_32_pcix(void);
void m68k_op_movep_re_16(void);
void m68k_op_movep_re_32(void);
void m68k_op_movep_er_16(void);
void m68k_op_movep_er_32(void);
void m68k_op_moves_8_ai(void);
void m68k_op_moves_8_pi(void);
void m68k_op_moves_8_pi7(void);
void m68k_op_moves_8_pd(void);
void m68k_op_moves_8_pd7(void);
void m68k_op_moves_8_di(void);
void m68k_op_moves_8_ix(void);
void m68k_op_moves_8_aw(void);
void m68k_op_moves_8_al(void);
void m68k_op_moves_16_ai(void);
void m68k_op_moves_16_pi(void);
void m68k_op_moves_16_pd(void);
void m68k_op_moves_16_di(void);
void m68k_op_moves_16_ix(void);
void m68k_op_moves_16_aw(void);
void m68k_op_moves_16_al(void);
void m68k_op_moves_32_ai(void);
void m68k_op_moves_32_pi(void);
void m68k_op_moves_32_pd(void);
void m68k_op_moves_32_di(void);
void m68k_op_moves_32_ix(void);
void m68k_op_moves_32_aw(void);
void m68k_op_moves_32_al(void);
void m68k_op_moveq_32(void);
void m68k_op_muls_16_d(void);
void m68k_op_muls_16_ai(void);
void m68k_op_muls_16_pi(void);
void m68k_op_muls_16_pd(void);
void m68k_op_muls_16_di(void);
void m68k_op_muls_16_ix(void);
void m68k_op_muls_16_aw(void);
void m68k_op_muls_16_al(void);
void m68k_op_muls_16_pcdi(void);
void m68k_op_muls_16_pcix(void);
void m68k_op_muls_16_i(void);
void m68k_op_mulu_16_d(void);
void m68k_op_mulu_16_ai(void);
void m68k_op_mulu_16_pi(void);
void m68k_op_mulu_16_pd(void);
void m68k_op_mulu_16_di(void);
void m68k_op_mulu_16_ix(void);
void m68k_op_mulu_16_aw(void);
void m68k_op_mulu_16_al(void);
void m68k_op_mulu_16_pcdi(void);
void m68k_op_mulu_16_pcix(void);
void m68k_op_mulu_16_i(void);
void m68k_op_mull_32_d(void);
void m68k_op_mull_32_ai(void);
void m68k_op_mull_32_pi(void);
void m68k_op_mull_32_pd(void);
void m68k_op_mull_32_di(void);
void m68k_op_mull_32_ix(void);
void m68k_op_mull_32_aw(void);
void m68k_op_mull_32_al(void);
void m68k_op_mull_32_pcdi(void);
void m68k_op_mull_32_pcix(void);
void m68k_op_mull_32_i(void);
void m68k_op_nbcd_8_d(void);
void m68k_op_nbcd_8_ai(void);
void m68k_op_nbcd_8_pi(void);
void m68k_op_nbcd_8_pi7(void);
void m68k_op_nbcd_8_pd(void);
void m68k_op_nbcd_8_pd7(void);
void m68k_op_nbcd_8_di(void);
void m68k_op_nbcd_8_ix(void);
void m68k_op_nbcd_8_aw(void);
void m68k_op_nbcd_8_al(void);
void m68k_op_neg_8_d(void);
void m68k_op_neg_8_ai(void);
void m68k_op_neg_8_pi(void);
void m68k_op_neg_8_pi7(void);
void m68k_op_neg_8_pd(void);
void m68k_op_neg_8_pd7(void);
void m68k_op_neg_8_di(void);
void m68k_op_neg_8_ix(void);
void m68k_op_neg_8_aw(void);
void m68k_op_neg_8_al(void);
void m68k_op_neg_16_d(void);
void m68k_op_neg_16_ai(void);
void m68k_op_neg_16_pi(void);
void m68k_op_neg_16_pd(void);
void m68k_op_neg_16_di(void);
void m68k_op_neg_16_ix(void);
void m68k_op_neg_16_aw(void);
void m68k_op_neg_16_al(void);
void m68k_op_neg_32_d(void);
void m68k_op_neg_32_ai(void);
void m68k_op_neg_32_pi(void);
void m68k_op_neg_32_pd(void);
void m68k_op_neg_32_di(void);
void m68k_op_neg_32_ix(void);
void m68k_op_neg_32_aw(void);
void m68k_op_neg_32_al(void);
void m68k_op_negx_8_d(void);
void m68k_op_negx_8_ai(void);
void m68k_op_negx_8_pi(void);
void m68k_op_negx_8_pi7(void);
void m68k_op_negx_8_pd(void);
void m68k_op_negx_8_pd7(void);
void m68k_op_negx_8_di(void);
void m68k_op_negx_8_ix(void);
void m68k_op_negx_8_aw(void);
void m68k_op_negx_8_al(void);
void m68k_op_negx_16_d(void);
void m68k_op_negx_16_ai(void);
void m68k_op_negx_16_pi(void);
void m68k_op_negx_16_pd(void);
void m68k_op_negx_16_di(void);
void m68k_op_negx_16_ix(void);
void m68k_op_negx_16_aw(void);
void m68k_op_negx_16_al(void);
void m68k_op_negx_32_d(void);
void m68k_op_negx_32_ai(void);
void m68k_op_negx_32_pi(void);
void m68k_op_negx_32_pd(void);
void m68k_op_negx_32_di(void);
void m68k_op_negx_32_ix(void);
void m68k_op_negx_32_aw(void);
void m68k_op_negx_32_al(void);
void m68k_op_nop(void);
void m68k_op_not_8_d(void);
void m68k_op_not_8_ai(void);
void m68k_op_not_8_pi(void);
void m68k_op_not_8_pi7(void);
void m68k_op_not_8_pd(void);
void m68k_op_not_8_pd7(void);
void m68k_op_not_8_di(void);
void m68k_op_not_8_ix(void);
void m68k_op_not_8_aw(void);
void m68k_op_not_8_al(void);
void m68k_op_not_16_d(void);
void m68k_op_not_16_ai(void);
void m68k_op_not_16_pi(void);
void m68k_op_not_16_pd(void);
void m68k_op_not_16_di(void);
void m68k_op_not_16_ix(void);
void m68k_op_not_16_aw(void);
void m68k_op_not_16_al(void);
void m68k_op_not_32_d(void);
void m68k_op_not_32_ai(void);
void m68k_op_not_32_pi(void);
void m68k_op_not_32_pd(void);
void m68k_op_not_32_di(void);
void m68k_op_not_32_ix(void);
void m68k_op_not_32_aw(void);
void m68k_op_not_32_al(void);
void m68k_op_or_er_8_d(void);
void m68k_op_or_er_8_ai(void);
void m68k_op_or_er_8_pi(void);
void m68k_op_or_er_8_pi7(void);
void m68k_op_or_er_8_pd(void);
void m68k_op_or_er_8_pd7(void);
void m68k_op_or_er_8_di(void);
void m68k_op_or_er_8_ix(void);
void m68k_op_or_er_8_aw(void);
void m68k_op_or_er_8_al(void);
void m68k_op_or_er_8_pcdi(void);
void m68k_op_or_er_8_pcix(void);
void m68k_op_or_er_8_i(void);
void m68k_op_or_er_16_d(void);
void m68k_op_or_er_16_ai(void);
void m68k_op_or_er_16_pi(void);
void m68k_op_or_er_16_pd(void);
void m68k_op_or_er_16_di(void);
void m68k_op_or_er_16_ix(void);
void m68k_op_or_er_16_aw(void);
void m68k_op_or_er_16_al(void);
void m68k_op_or_er_16_pcdi(void);
void m68k_op_or_er_16_pcix(void);
void m68k_op_or_er_16_i(void);
void m68k_op_or_er_32_d(void);
void m68k_op_or_er_32_ai(void);
void m68k_op_or_er_32_pi(void);
void m68k_op_or_er_32_pd(void);
void m68k_op_or_er_32_di(void);
void m68k_op_or_er_32_ix(void);
void m68k_op_or_er_32_aw(void);
void m68k_op_or_er_32_al(void);
void m68k_op_or_er_32_pcdi(void);
void m68k_op_or_er_32_pcix(void);
void m68k_op_or_er_32_i(void);
void m68k_op_or_re_8_ai(void);
void m68k_op_or_re_8_pi(void);
void m68k_op_or_re_8_pi7(void);
void m68k_op_or_re_8_pd(void);
void m68k_op_or_re_8_pd7(void);
void m68k_op_or_re_8_di(void);
void m68k_op_or_re_8_ix(void);
void m68k_op_or_re_8_aw(void);
void m68k_op_or_re_8_al(void);
void m68k_op_or_re_16_ai(void);
void m68k_op_or_re_16_pi(void);
void m68k_op_or_re_16_pd(void);
void m68k_op_or_re_16_di(void);
void m68k_op_or_re_16_ix(void);
void m68k_op_or_re_16_aw(void);
void m68k_op_or_re_16_al(void);
void m68k_op_or_re_32_ai(void);
void m68k_op_or_re_32_pi(void);
void m68k_op_or_re_32_pd(void);
void m68k_op_or_re_32_di(void);
void m68k_op_or_re_32_ix(void);
void m68k_op_or_re_32_aw(void);
void m68k_op_or_re_32_al(void);
void m68k_op_ori_8_d(void);
void m68k_op_ori_8_ai(void);
void m68k_op_ori_8_pi(void);
void m68k_op_ori_8_pi7(void);
void m68k_op_ori_8_pd(void);
void m68k_op_ori_8_pd7(void);
void m68k_op_ori_8_di(void);
void m68k_op_ori_8_ix(void);
void m68k_op_ori_8_aw(void);
void m68k_op_ori_8_al(void);
void m68k_op_ori_16_d(void);
void m68k_op_ori_16_ai(void);
void m68k_op_ori_16_pi(void);
void m68k_op_ori_16_pd(void);
void m68k_op_ori_16_di(void);
void m68k_op_ori_16_ix(void);
void m68k_op_ori_16_aw(void);
void m68k_op_ori_16_al(void);
void m68k_op_ori_32_d(void);
void m68k_op_ori_32_ai(void);
void m68k_op_ori_32_pi(void);
void m68k_op_ori_32_pd(void);
void m68k_op_ori_32_di(void);
void m68k_op_ori_32_ix(void);
void m68k_op_ori_32_aw(void);
void m68k_op_ori_32_al(void);
void m68k_op_ori_to_ccr_16(void);
void m68k_op_ori_to_sr_16(void);
void m68k_op_pack_rr_16(void);
void m68k_op_pack_mm_16_ax7(void);
void m68k_op_pack_mm_16_ay7(void);
void m68k_op_pack_mm_16_axy7(void);
void m68k_op_pack_mm_16(void);
void m68k_op_pea_32_ai(void);
void m68k_op_pea_32_di(void);
void m68k_op_pea_32_ix(void);
void m68k_op_pea_32_aw(void);
void m68k_op_pea_32_al(void);
void m68k_op_pea_32_pcdi(void);
void m68k_op_pea_32_pcix(void);
void m68k_op_reset(void);
void m68k_op_ror_s_8(void);
void m68k_op_ror_s_16(void);
void m68k_op_ror_s_32(void);
void m68k_op_ror_r_8(void);
void m68k_op_ror_r_16(void);
void m68k_op_ror_r_32(void);
void m68k_op_ror_16_ai(void);
void m68k_op_ror_16_pi(void);
void m68k_op_ror_16_pd(void);
void m68k_op_ror_16_di(void);
void m68k_op_ror_16_ix(void);
void m68k_op_ror_16_aw(void);
void m68k_op_ror_16_al(void);
void m68k_op_rol_s_8(void);
void m68k_op_rol_s_16(void);
void m68k_op_rol_s_32(void);
void m68k_op_rol_r_8(void);
void m68k_op_rol_r_16(void);
void m68k_op_rol_r_32(void);
void m68k_op_rol_16_ai(void);
void m68k_op_rol_16_pi(void);
void m68k_op_rol_16_pd(void);
void m68k_op_rol_16_di(void);
void m68k_op_rol_16_ix(void);
void m68k_op_rol_16_aw(void);
void m68k_op_rol_16_al(void);
void m68k_op_roxr_s_8(void);
void m68k_op_roxr_s_16(void);
void m68k_op_roxr_s_32(void);
void m68k_op_roxr_r_8(void);
void m68k_op_roxr_r_16(void);
void m68k_op_roxr_r_32(void);
void m68k_op_roxr_16_ai(void);
void m68k_op_roxr_16_pi(void);
void m68k_op_roxr_16_pd(void);
void m68k_op_roxr_16_di(void);
void m68k_op_roxr_16_ix(void);
void m68k_op_roxr_16_aw(void);
void m68k_op_roxr_16_al(void);
void m68k_op_roxl_s_8(void);
void m68k_op_roxl_s_16(void);
void m68k_op_roxl_s_32(void);
void m68k_op_roxl_r_8(void);
void m68k_op_roxl_r_16(void);
void m68k_op_roxl_r_32(void);
void m68k_op_roxl_16_ai(void);
void m68k_op_roxl_16_pi(void);
void m68k_op_roxl_16_pd(void);
void m68k_op_roxl_16_di(void);
void m68k_op_roxl_16_ix(void);
void m68k_op_roxl_16_aw(void);
void m68k_op_roxl_16_al(void);
void m68k_op_rtd_32(void);
void m68k_op_rte_32(void);
void m68k_op_rtm_32(void);
void m68k_op_rtr_32(void);
void m68k_op_rts_32(void);
void m68k_op_sbcd_rr_8(void);
void m68k_op_sbcd_mm_8_ax7(void);
void m68k_op_sbcd_mm_8_ay7(void);
void m68k_op_sbcd_mm_8_axy7(void);
void m68k_op_sbcd_mm_8(void);
void m68k_op_st_8_d(void);
void m68k_op_st_8_ai(void);
void m68k_op_st_8_pi(void);
void m68k_op_st_8_pi7(void);
void m68k_op_st_8_pd(void);
void m68k_op_st_8_pd7(void);
void m68k_op_st_8_di(void);
void m68k_op_st_8_ix(void);
void m68k_op_st_8_aw(void);
void m68k_op_st_8_al(void);
void m68k_op_sf_8_d(void);
void m68k_op_sf_8_ai(void);
void m68k_op_sf_8_pi(void);
void m68k_op_sf_8_pi7(void);
void m68k_op_sf_8_pd(void);
void m68k_op_sf_8_pd7(void);
void m68k_op_sf_8_di(void);
void m68k_op_sf_8_ix(void);
void m68k_op_sf_8_aw(void);
void m68k_op_sf_8_al(void);
void m68k_op_shi_8_d(void);
void m68k_op_sls_8_d(void);
void m68k_op_scc_8_d(void);
void m68k_op_scs_8_d(void);
void m68k_op_sne_8_d(void);
void m68k_op_seq_8_d(void);
void m68k_op_svc_8_d(void);
void m68k_op_svs_8_d(void);
void m68k_op_spl_8_d(void);
void m68k_op_smi_8_d(void);
void m68k_op_sge_8_d(void);
void m68k_op_slt_8_d(void);
void m68k_op_sgt_8_d(void);
void m68k_op_sle_8_d(void);
void m68k_op_shi_8_ai(void);
void m68k_op_shi_8_pi(void);
void m68k_op_shi_8_pi7(void);
void m68k_op_shi_8_pd(void);
void m68k_op_shi_8_pd7(void);
void m68k_op_shi_8_di(void);
void m68k_op_shi_8_ix(void);
void m68k_op_shi_8_aw(void);
void m68k_op_shi_8_al(void);
void m68k_op_sls_8_ai(void);
void m68k_op_sls_8_pi(void);
void m68k_op_sls_8_pi7(void);
void m68k_op_sls_8_pd(void);
void m68k_op_sls_8_pd7(void);
void m68k_op_sls_8_di(void);
void m68k_op_sls_8_ix(void);
void m68k_op_sls_8_aw(void);
void m68k_op_sls_8_al(void);
void m68k_op_scc_8_ai(void);
void m68k_op_scc_8_pi(void);
void m68k_op_scc_8_pi7(void);
void m68k_op_scc_8_pd(void);
void m68k_op_scc_8_pd7(void);
void m68k_op_scc_8_di(void);
void m68k_op_scc_8_ix(void);
void m68k_op_scc_8_aw(void);
void m68k_op_scc_8_al(void);
void m68k_op_scs_8_ai(void);
void m68k_op_scs_8_pi(void);
void m68k_op_scs_8_pi7(void);
void m68k_op_scs_8_pd(void);
void m68k_op_scs_8_pd7(void);
void m68k_op_scs_8_di(void);
void m68k_op_scs_8_ix(void);
void m68k_op_scs_8_aw(void);
void m68k_op_scs_8_al(void);
void m68k_op_sne_8_ai(void);
void m68k_op_sne_8_pi(void);
void m68k_op_sne_8_pi7(void);
void m68k_op_sne_8_pd(void);
void m68k_op_sne_8_pd7(void);
void m68k_op_sne_8_di(void);
void m68k_op_sne_8_ix(void);
void m68k_op_sne_8_aw(void);
void m68k_op_sne_8_al(void);
void m68k_op_seq_8_ai(void);
void m68k_op_seq_8_pi(void);
void m68k_op_seq_8_pi7(void);
void m68k_op_seq_8_pd(void);
void m68k_op_seq_8_pd7(void);
void m68k_op_seq_8_di(void);
void m68k_op_seq_8_ix(void);
void m68k_op_seq_8_aw(void);
void m68k_op_seq_8_al(void);
void m68k_op_svc_8_ai(void);
void m68k_op_svc_8_pi(void);
void m68k_op_svc_8_pi7(void);
void m68k_op_svc_8_pd(void);
void m68k_op_svc_8_pd7(void);
void m68k_op_svc_8_di(void);
void m68k_op_svc_8_ix(void);
void m68k_op_svc_8_aw(void);
void m68k_op_svc_8_al(void);
void m68k_op_svs_8_ai(void);
void m68k_op_svs_8_pi(void);
void m68k_op_svs_8_pi7(void);
void m68k_op_svs_8_pd(void);
void m68k_op_svs_8_pd7(void);
void m68k_op_svs_8_di(void);
void m68k_op_svs_8_ix(void);
void m68k_op_svs_8_aw(void);
void m68k_op_svs_8_al(void);
void m68k_op_spl_8_ai(void);
void m68k_op_spl_8_pi(void);
void m68k_op_spl_8_pi7(void);
void m68k_op_spl_8_pd(void);
void m68k_op_spl_8_pd7(void);
void m68k_op_spl_8_di(void);
void m68k_op_spl_8_ix(void);
void m68k_op_spl_8_aw(void);
void m68k_op_spl_8_al(void);
void m68k_op_smi_8_ai(void);
void m68k_op_smi_8_pi(void);
void m68k_op_smi_8_pi7(void);
void m68k_op_smi_8_pd(void);
void m68k_op_smi_8_pd7(void);
void m68k_op_smi_8_di(void);
void m68k_op_smi_8_ix(void);
void m68k_op_smi_8_aw(void);
void m68k_op_smi_8_al(void);
void m68k_op_sge_8_ai(void);
void m68k_op_sge_8_pi(void);
void m68k_op_sge_8_pi7(void);
void m68k_op_sge_8_pd(void);
void m68k_op_sge_8_pd7(void);
void m68k_op_sge_8_di(void);
void m68k_op_sge_8_ix(void);
void m68k_op_sge_8_aw(void);
void m68k_op_sge_8_al(void);
void m68k_op_slt_8_ai(void);
void m68k_op_slt_8_pi(void);
void m68k_op_slt_8_pi7(void);
void m68k_op_slt_8_pd(void);
void m68k_op_slt_8_pd7(void);
void m68k_op_slt_8_di(void);
void m68k_op_slt_8_ix(void);
void m68k_op_slt_8_aw(void);
void m68k_op_slt_8_al(void);
void m68k_op_sgt_8_ai(void);
void m68k_op_sgt_8_pi(void);
void m68k_op_sgt_8_pi7(void);
void m68k_op_sgt_8_pd(void);
void m68k_op_sgt_8_pd7(void);
void m68k_op_sgt_8_di(void);
void m68k_op_sgt_8_ix(void);
void m68k_op_sgt_8_aw(void);
void m68k_op_sgt_8_al(void);
void m68k_op_sle_8_ai(void);
void m68k_op_sle_8_pi(void);
void m68k_op_sle_8_pi7(void);
void m68k_op_sle_8_pd(void);
void m68k_op_sle_8_pd7(void);
void m68k_op_sle_8_di(void);
void m68k_op_sle_8_ix(void);
void m68k_op_sle_8_aw(void);
void m68k_op_sle_8_al(void);
void m68k_op_stop(void);
void m68k_op_sub_er_8_d(void);
void m68k_op_sub_er_8_ai(void);
void m68k_op_sub_er_8_pi(void);
void m68k_op_sub_er_8_pi7(void);
void m68k_op_sub_er_8_pd(void);
void m68k_op_sub_er_8_pd7(void);
void m68k_op_sub_er_8_di(void);
void m68k_op_sub_er_8_ix(void);
void m68k_op_sub_er_8_aw(void);
void m68k_op_sub_er_8_al(void);
void m68k_op_sub_er_8_pcdi(void);
void m68k_op_sub_er_8_pcix(void);
void m68k_op_sub_er_8_i(void);
void m68k_op_sub_er_16_d(void);
void m68k_op_sub_er_16_a(void);
void m68k_op_sub_er_16_ai(void);
void m68k_op_sub_er_16_pi(void);
void m68k_op_sub_er_16_pd(void);
void m68k_op_sub_er_16_di(void);
void m68k_op_sub_er_16_ix(void);
void m68k_op_sub_er_16_aw(void);
void m68k_op_sub_er_16_al(void);
void m68k_op_sub_er_16_pcdi(void);
void m68k_op_sub_er_16_pcix(void);
void m68k_op_sub_er_16_i(void);
void m68k_op_sub_er_32_d(void);
void m68k_op_sub_er_32_a(void);
void m68k_op_sub_er_32_ai(void);
void m68k_op_sub_er_32_pi(void);
void m68k_op_sub_er_32_pd(void);
void m68k_op_sub_er_32_di(void);
void m68k_op_sub_er_32_ix(void);
void m68k_op_sub_er_32_aw(void);
void m68k_op_sub_er_32_al(void);
void m68k_op_sub_er_32_pcdi(void);
void m68k_op_sub_er_32_pcix(void);
void m68k_op_sub_er_32_i(void);
void m68k_op_sub_re_8_ai(void);
void m68k_op_sub_re_8_pi(void);
void m68k_op_sub_re_8_pi7(void);
void m68k_op_sub_re_8_pd(void);
void m68k_op_sub_re_8_pd7(void);
void m68k_op_sub_re_8_di(void);
void m68k_op_sub_re_8_ix(void);
void m68k_op_sub_re_8_aw(void);
void m68k_op_sub_re_8_al(void);
void m68k_op_sub_re_16_ai(void);
void m68k_op_sub_re_16_pi(void);
void m68k_op_sub_re_16_pd(void);
void m68k_op_sub_re_16_di(void);
void m68k_op_sub_re_16_ix(void);
void m68k_op_sub_re_16_aw(void);
void m68k_op_sub_re_16_al(void);
void m68k_op_sub_re_32_ai(void);
void m68k_op_sub_re_32_pi(void);
void m68k_op_sub_re_32_pd(void);
void m68k_op_sub_re_32_di(void);
void m68k_op_sub_re_32_ix(void);
void m68k_op_sub_re_32_aw(void);
void m68k_op_sub_re_32_al(void);
void m68k_op_suba_16_d(void);
void m68k_op_suba_16_a(void);
void m68k_op_suba_16_ai(void);
void m68k_op_suba_16_pi(void);
void m68k_op_suba_16_pd(void);
void m68k_op_suba_16_di(void);
void m68k_op_suba_16_ix(void);
void m68k_op_suba_16_aw(void);
void m68k_op_suba_16_al(void);
void m68k_op_suba_16_pcdi(void);
void m68k_op_suba_16_pcix(void);
void m68k_op_suba_16_i(void);
void m68k_op_suba_32_d(void);
void m68k_op_suba_32_a(void);
void m68k_op_suba_32_ai(void);
void m68k_op_suba_32_pi(void);
void m68k_op_suba_32_pd(void);
void m68k_op_suba_32_di(void);
void m68k_op_suba_32_ix(void);
void m68k_op_suba_32_aw(void);
void m68k_op_suba_32_al(void);
void m68k_op_suba_32_pcdi(void);
void m68k_op_suba_32_pcix(void);
void m68k_op_suba_32_i(void);
void m68k_op_subi_8_d(void);
void m68k_op_subi_8_ai(void);
void m68k_op_subi_8_pi(void);
void m68k_op_subi_8_pi7(void);
void m68k_op_subi_8_pd(void);
void m68k_op_subi_8_pd7(void);
void m68k_op_subi_8_di(void);
void m68k_op_subi_8_ix(void);
void m68k_op_subi_8_aw(void);
void m68k_op_subi_8_al(void);
void m68k_op_subi_16_d(void);
void m68k_op_subi_16_ai(void);
void m68k_op_subi_16_pi(void);
void m68k_op_subi_16_pd(void);
void m68k_op_subi_16_di(void);
void m68k_op_subi_16_ix(void);
void m68k_op_subi_16_aw(void);
void m68k_op_subi_16_al(void);
void m68k_op_subi_32_d(void);
void m68k_op_subi_32_ai(void);
void m68k_op_subi_32_pi(void);
void m68k_op_subi_32_pd(void);
void m68k_op_subi_32_di(void);
void m68k_op_subi_32_ix(void);
void m68k_op_subi_32_aw(void);
void m68k_op_subi_32_al(void);
void m68k_op_subq_8_d(void);
void m68k_op_subq_8_ai(void);
void m68k_op_subq_8_pi(void);
void m68k_op_subq_8_pi7(void);
void m68k_op_subq_8_pd(void);
void m68k_op_subq_8_pd7(void);
void m68k_op_subq_8_di(void);
void m68k_op_subq_8_ix(void);
void m68k_op_subq_8_aw(void);
void m68k_op_subq_8_al(void);
void m68k_op_subq_16_d(void);
void m68k_op_subq_16_a(void);
void m68k_op_subq_16_ai(void);
void m68k_op_subq_16_pi(void);
void m68k_op_subq_16_pd(void);
void m68k_op_subq_16_di(void);
void m68k_op_subq_16_ix(void);
void m68k_op_subq_16_aw(void);
void m68k_op_subq_16_al(void);
void m68k_op_subq_32_d(void);
void m68k_op_subq_32_a(void);
void m68k_op_subq_32_ai(void);
void m68k_op_subq_32_pi(void);
void m68k_op_subq_32_pd(void);
void m68k_op_subq_32_di(void);
void m68k_op_subq_32_ix(void);
void m68k_op_subq_32_aw(void);
void m68k_op_subq_32_al(void);
void m68k_op_subx_rr_8(void);
void m68k_op_subx_rr_16(void);
void m68k_op_subx_rr_32(void);
void m68k_op_subx_mm_8_ax7(void);
void m68k_op_subx_mm_8_ay7(void);
void m68k_op_subx_mm_8_axy7(void);
void m68k_op_subx_mm_8(void);
void m68k_op_subx_mm_16(void);
void m68k_op_subx_mm_32(void);
void m68k_op_swap_32(void);
void m68k_op_tas_8_d(void);
void m68k_op_tas_8_ai(void);
void m68k_op_tas_8_pi(void);
void m68k_op_tas_8_pi7(void);
void m68k_op_tas_8_pd(void);
void m68k_op_tas_8_pd7(void);
void m68k_op_tas_8_di(void);
void m68k_op_tas_8_ix(void);
void m68k_op_tas_8_aw(void);
void m68k_op_tas_8_al(void);
void m68k_op_trap(void);
void m68k_op_trapt(void);
void m68k_op_trapt_16(void);
void m68k_op_trapt_32(void);
void m68k_op_trapf(void);
void m68k_op_trapf_16(void);
void m68k_op_trapf_32(void);
void m68k_op_traphi(void);
void m68k_op_trapls(void);
void m68k_op_trapcc(void);
void m68k_op_trapcs(void);
void m68k_op_trapne(void);
void m68k_op_trapeq(void);
void m68k_op_trapvc(void);
void m68k_op_trapvs(void);
void m68k_op_trappl(void);
void m68k_op_trapmi(void);
void m68k_op_trapge(void);
void m68k_op_traplt(void);
void m68k_op_trapgt(void);
void m68k_op_traple(void);
void m68k_op_traphi_16(void);
void m68k_op_trapls_16(void);
void m68k_op_trapcc_16(void);
void m68k_op_trapcs_16(void);
void m68k_op_trapne_16(void);
void m68k_op_trapeq_16(void);
void m68k_op_trapvc_16(void);
void m68k_op_trapvs_16(void);
void m68k_op_trappl_16(void);
void m68k_op_trapmi_16(void);
void m68k_op_trapge_16(void);
void m68k_op_traplt_16(void);
void m68k_op_trapgt_16(void);
void m68k_op_traple_16(void);
void m68k_op_traphi_32(void);
void m68k_op_trapls_32(void);
void m68k_op_trapcc_32(void);
void m68k_op_trapcs_32(void);
void m68k_op_trapne_32(void);
void m68k_op_trapeq_32(void);
void m68k_op_trapvc_32(void);
void m68k_op_trapvs_32(void);
void m68k_op_trappl_32(void);
void m68k_op_trapmi_32(void);
void m68k_op_trapge_32(void);
void m68k_op_traplt_32(void);
void m68k_op_trapgt_32(void);
void m68k_op_traple_32(void);
void m68k_op_trapv(void);
void m68k_op_tst_8_d(void);
void m68k_op_tst_8_ai(void);
void m68k_op_tst_8_pi(void);
void m68k_op_tst_8_pi7(void);
void m68k_op_tst_8_pd(void);
void m68k_op_tst_8_pd7(void);
void m68k_op_tst_8_di(void);
void m68k_op_tst_8_ix(void);
void m68k_op_tst_8_aw(void);
void m68k_op_tst_8_al(void);
void m68k_op_tst_8_pcdi(void);
void m68k_op_tst_8_pcix(void);
void m68k_op_tst_8_i(void);
void m68k_op_tst_16_d(void);
void m68k_op_tst_16_a(void);
void m68k_op_tst_16_ai(void);
void m68k_op_tst_16_pi(void);
void m68k_op_tst_16_pd(void);
void m68k_op_tst_16_di(void);
void m68k_op_tst_16_ix(void);
void m68k_op_tst_16_aw(void);
void m68k_op_tst_16_al(void);
void m68k_op_tst_16_pcdi(void);
void m68k_op_tst_16_pcix(void);
void m68k_op_tst_16_i(void);
void m68k_op_tst_32_d(void);
void m68k_op_tst_32_a(void);
void m68k_op_tst_32_ai(void);
void m68k_op_tst_32_pi(void);
void m68k_op_tst_32_pd(void);
void m68k_op_tst_32_di(void);
void m68k_op_tst_32_ix(void);
void m68k_op_tst_32_aw(void);
void m68k_op_tst_32_al(void);
void m68k_op_tst_32_pcdi(void);
void m68k_op_tst_32_pcix(void);
void m68k_op_tst_32_i(void);
void m68k_op_unlk_32_a7(void);
void m68k_op_unlk_32(void);
void m68k_op_unpk_rr_16(void);
void m68k_op_unpk_mm_16_ax7(void);
void m68k_op_unpk_mm_16_ay7(void);
void m68k_op_unpk_mm_16_axy7(void);
void m68k_op_unpk_mm_16(void);
/* Build the opcode handler table */
void m68ki_build_opcode_table(void);

extern void (*m68ki_instruction_jump_table[0x10000])(void); /* opcode handler jump table */
extern unsigned char m68ki_cycles[][0x10000];


/* ======================================================================== */
/* ============================== END OF FILE ============================= */
/* ======================================================================== */

#endif /* M68KOPS__HEADER */


