#define KEY_ESC             1                      
#define KEY_1               2                      
#define KEY_2               3                      
#define KEY_3               4                      
#define KEY_4               5                      
#define KEY_5               6                      
#define KEY_6               7                      
#define KEY_7               8                      
#define KEY_8               9                      
#define KEY_9              10                      
#define KEY_0              11                      
#define KEY_MINUS          12                      
#define KEY_EQUALS         13                      
#define KEY_BACKSPACE      14                      
#define KEY_TAB            15                      
#define KEY_Q              16                      
#define KEY_W              17                      
#define KEY_E              18                      
#define KEY_R              19                      
#define KEY_T              20                      
#define KEY_Y              21                      
#define KEY_U              22                      
#define KEY_I              23                      
#define KEY_O              24                      
#define KEY_P              25                      
#define KEY_OPENBRACE      26                      
#define KEY_CLOSEBRACE     27                      
#define KEY_ENTER          28                      
#define KEY_LCONTROL       29                      
#define KEY_A              30                      
#define KEY_S              31                      
#define KEY_D              32                      
#define KEY_F              33                      
#define KEY_G              34                      
#define KEY_H              35                      
#define KEY_J              36                      
#define KEY_K              37                      
#define KEY_L              38                      
#define KEY_COLON          39                      
#define KEY_QUOTE          40                      
#define KEY_TILDE          41                      
#define KEY_LSHIFT         42                      
#define KEY_BACKSLASH      43                      
#define KEY_Z              44                      
#define KEY_X              45                      
#define KEY_C              46                      
#define KEY_V              47                      
#define KEY_B              48                      
#define KEY_N              49                      
#define KEY_M              50                      
#define KEY_COMMA          51                      
#define KEY_STOP           52                      
#define KEY_SLASH          53                      
#define KEY_RSHIFT         54                      
#define KEY_ASTERISK       55                      
#define KEY_ALT            56                      
#define KEY_SPACE          57                      
#define KEY_CAPSLOCK       58                      
#define KEY_F1             59                      
#define KEY_F2             60                      
#define KEY_F3             61                      
#define KEY_F4             62                      
#define KEY_F5             63                      
#define KEY_F6             64                      
#define KEY_F7             65                      
#define KEY_F8             66                      
#define KEY_F9             67                      
#define KEY_F10            68                      
#define KEY_NUMLOCK        69                      
#define KEY_SCRLOCK        70                      
#define KEY_HOME           71                      
#define KEY_UP             72                      
#define KEY_PGUP           73                      
#define KEY_MINUS_PAD      74                      
#define KEY_LEFT           75                      
#define KEY_5_PAD          76                      
#define KEY_RIGHT          77                      
#define KEY_PLUS_PAD       78                      
#define KEY_END            79                      
#define KEY_DOWN           80                      
#define KEY_PGDN           81                      
#define KEY_INSERT         82                      
#define KEY_DEL            83                      
#define KEY_PRTSCR         84                      
#define KEY_F11            87                      
#define KEY_F12            88                      
#define KEY_DEL_PAD        90                      
#define KEY_LWIN           91                      
#define KEY_RWIN           92                      
#define KEY_MENU           93                      
#define KEY_1_PAD	  101                      
#define KEY_2_PAD	  102                      
#define KEY_3_PAD	  103                      
#define KEY_4_PAD	  104                      
#define KEY_6_PAD	  106                      
#define KEY_7_PAD	  107                      
#define KEY_8_PAD	  108                      
#define KEY_9_PAD	  109                      
#define KEY_0_PAD	  110                      
#define KEY_BACKSLASH2    111 /* 71? */            
#define KEY_ENTER_PAD	  115                      
#define KEY_RCONTROL      120                      
#define KEY_ALTGR         121                      
#define KEY_SLASH_PAD     122                      
#define KEY_PAUSE         123                      

#define KEY_MAX		  128

#define JOY_TYPE_AUTODETECT -1
#define JOY_TYPE_NONE 	    0
#define JOY_TYPE_STANDARD   1
#define JOY_TYPE_2PADS      2
#define JOY_TYPE_4BUTTON    3
#define JOY_TYPE_6BUTTON    4
#define JOY_TYPE_8BUTTON    5
#define JOY_TYPE_FSPRO      6
#define JOY_TYPE_WINGEX     7
#define JOY_TYPE_SIDEWINDER 8
#define JOY_TYPE_GAMEPAD_PRO 9
#define JOY_TYPE_GRIP 	    10
#define JOY_TYPE_GRIP4 	    11
#define JOY_TYPE_SNESPAD_LPT1 12
#define JOY_TYPE_SNESPAD_LPT2 13
#define JOY_TYPE_SNESPAD_LPT3 14
#define JOY_TYPE_PSXPAD_LPT1 15
#define JOY_TYPE_PSXPAD_LPT2 16
#define JOY_TYPE_PSXPAD_LPT3 17
#define JOY_TYPE_N64PAD_LPT1 18
#define JOY_TYPE_N64PAD_LPT2 19
#define JOY_TYPE_N64PAD_LPT3 20
#define JOY_TYPE_WINGWARRIOR 21
#define JOY_TYPE_IFSEGA_ISA 22
#define JOY_TYPE_IFSEGA_PCI 23

#define GFX_AUTO 0
#define GFX_VESA1 1
#define GFX_VESA2B 2
#define GFX_VESA2L 3
#define GFX_VESA3 4
